'use strict';

const shared = require('@vue/shared');
const compilerCore = require('@vue/compiler-core');
const fs = require('node:fs');
const qinjsCliShared = require('@fe6/qinjs-cli-shared');
const babelGenerate = require('@babel/generator');
const types = require('@babel/types');
const parser = require('@babel/parser');
const estreeWalker = require('estree-walker');
const qinjsShared = require('@fe6/qinjs-shared');
const compilerDom = require('@vue/compiler-dom');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const fs__default = /*#__PURE__*/_interopDefaultCompat(fs);
const babelGenerate__default = /*#__PURE__*/_interopDefaultCompat(babelGenerate);

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class IdentifierGenerator {
  constructor() {
    // u 被框架占用了，不提供 u 开头的变量名
    __publicField(this, "_chars", "abcdefghijklmnopqrstvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
    __publicField(this, "_nextIds", [0]);
  }
  next() {
    const r = [];
    for (const char of this._nextIds) {
      r.unshift(this._chars[char]);
    }
    this._increment();
    const id = r.join("");
    if (keywords.includes(id)) {
      return this.next();
    }
    return id;
  }
  _increment() {
    for (let i = 0; i < this._nextIds.length; i++) {
      const val = ++this._nextIds[i];
      if (val >= this._chars.length) {
        this._nextIds[i] = 0;
      } else {
        return;
      }
    }
    this._nextIds.push(0);
  }
  *[Symbol.iterator]() {
    while (true) {
      yield this.next();
    }
  }
}
const keywords = [
  "abstract",
  "arguments",
  "await",
  "boolean",
  "break",
  "byte",
  "case",
  "catch",
  "char",
  "class",
  "const",
  "continue",
  "debugger",
  "default",
  "delete",
  "do",
  "double",
  "else",
  "enum",
  "eval",
  "export",
  "extends",
  "false",
  "final",
  "finally",
  "float",
  "for",
  "function",
  "goto",
  "if",
  "implements",
  "import",
  "in",
  "instanceof",
  "int",
  "interface",
  "let",
  "long",
  "native",
  "new",
  "null",
  "package",
  "private",
  "protected",
  "public",
  "return",
  "short",
  "static",
  "super",
  "switch",
  "synchronized",
  "this",
  "throw",
  "throws",
  "transient",
  "true",
  "try",
  "typeof",
  "var",
  "void",
  "volatile",
  "while",
  "with",
  "yield"
];

const V_ON = Symbol(`vOn`);
const V_FOR = Symbol(`vFor`);
const EXTEND = Symbol(`extend`);
const SET_REF = Symbol(`setRef`);
const CAMELIZE = Symbol(`camelize`);
const HYPHENATE = Symbol(`hyphenate`);
const RENDER_PROPS = Symbol(`renderProps`);
const RENDER_SLOT = Symbol(`renderSlot`);
const DYNAMIC_SLOT = Symbol(`dynamicSlot`);
const WITH_SCOPED_SLOT = Symbol(`withScopedSlot`);
const STRINGIFY_STYLE = Symbol(`stringifyStyle`);
const NORMALIZE_CLASS = Symbol(`normalizeClass`);
const TO_DISPLAY_STRING = Symbol(`toDisplayString`);
const WITH_MODEL_MODIFIERS = Symbol(`withModelModifiers`);
compilerCore.registerRuntimeHelpers({
  [V_ON]: "o",
  [V_FOR]: "f",
  [EXTEND]: "e",
  [SET_REF]: "sr",
  [CAMELIZE]: "c",
  [HYPHENATE]: "h",
  [RENDER_PROPS]: "p",
  [RENDER_SLOT]: "r",
  [DYNAMIC_SLOT]: "d",
  [WITH_SCOPED_SLOT]: "w",
  [STRINGIFY_STYLE]: "s",
  [NORMALIZE_CLASS]: "n",
  [TO_DISPLAY_STRING]: "t",
  [WITH_MODEL_MODIFIERS]: "m",
  [qinjsCliShared.STRINGIFY_JSON]: "j"
});

function createObjectExpression(properties) {
  return types.objectExpression(properties);
}
function createVIfConditionalExpression({
  condition,
  properties
}) {
  return types.conditionalExpression(
    condition,
    types.objectExpression(properties),
    types.objectExpression([])
  );
}
function createVIfSpreadElement(vIfScope) {
  return types.spreadElement(createVIfConditionalExpression(vIfScope));
}
function parseExpr(code, context, node) {
  if (!shared.isString(code)) {
    node = code;
    code = genExpr(code);
  }
  try {
    return parser.parseExpression(code, {
      plugins: context.expressionPlugins
    });
  } catch (e) {
    context.onError(
      compilerCore.createCompilerError(
        45,
        node && node.loc,
        void 0,
        "\n" + code + "\n" + e.message
      )
    );
  }
}
function parseParam(code, context, node) {
  const {
    params: [expr]
  } = parseExpr(`(${code})=>{}`, context, node);
  return expr;
}
function isUndefined(expr) {
  return types.isIdentifier(expr) && expr.name === "undefined";
}
function isTrueExpr(expr) {
  if (types.isNullLiteral(expr)) {
    return false;
  }
  if (types.isStringLiteral(expr) || types.isNumericLiteral(expr) || types.isBooleanLiteral(expr) || types.isBigIntLiteral(expr) || types.isDecimalLiteral(expr)) {
    return !!expr.value;
  }
  return true;
}
function parseStringLiteral(expr) {
  if (types.isIdentifier(expr)) {
    return types.stringLiteral(expr.name);
  }
  if (types.isStringLiteral(expr)) {
    return types.stringLiteral(expr.value);
  }
  return types.stringLiteral("");
}

const ATTR_VUE_ID = "u-i";
const ATTR_VUE_SLOTS = "u-s";
const ATTR_VUE_PROPS = "u-p";
const ATTR_VUE_REF = "u-" + qinjsCliShared.VUE_REF;
const ATTR_VUE_REF_IN_FOR = "u-" + qinjsCliShared.VUE_REF_IN_FOR;
const ATTR_COM_TYPE = "u-t";
const SCOPED_SLOT_IDENTIFIER = "__SCOPED_SLOT__";
const VIRTUAL_HOST_STYLE = "virtualHostStyle";
const VIRTUAL_HOST_CLASS = "virtualHostClass";
function rewriteSpreadElement(name, expr, loc, context) {
  return rewirteWithHelper(name, expr.argument, loc, context);
}
function rewirteWithHelper(name, expr, loc, context) {
  return parseExprWithRewrite(
    context.helperString(name) + "(" + genBabelExpr(expr) + ")",
    loc,
    context
  );
}
function parseExprWithRewrite(code, loc, context, node) {
  return parseExpr(
    rewriteExpression(compilerCore.createSimpleExpression(code, false, loc), context, node),
    context
  );
}
function parseExprWithRewriteClass(code, loc, context, node) {
  return parseExpr(
    rewriteExpression(
      compilerCore.createSimpleExpression(code, false, loc),
      context,
      !isUndefined(node) ? types.conditionalExpression(node, types.numericLiteral(1), types.stringLiteral("")) : node
    ),
    context
  );
}
function rewriteExpression(node, context, babelNode, scope = context.currentScope, {
  property,
  ignoreLiteral,
  referencedScope
} = {
  property: true,
  ignoreLiteral: false
}) {
  if (node.type === 4 && node.isStatic) {
    return node;
  }
  if (!babelNode) {
    const code = genExpr(node);
    babelNode = parseExpr(code, context, node);
    if (!babelNode) {
      return compilerCore.createSimpleExpression(code);
    }
  }
  if (!ignoreLiteral && isStaticLiteral(babelNode)) {
    return node;
  }
  if (isUndefined(babelNode)) {
    return compilerCore.createSimpleExpression("undefined", false, node.loc);
  }
  if (context.filters?.length) {
    if (isReferencedByIds(babelNode, context.filters)) {
      return compilerCore.createSimpleExpression(genExpr(node), false, node.loc);
    }
  }
  referencedScope = referencedScope || findReferencedScope(babelNode, scope);
  const id = referencedScope.id.next();
  if (property) {
    referencedScope.properties.push(
      types.objectProperty(types.identifier(id), babelNode)
    );
  }
  if (isVIfScope(referencedScope)) {
    if (isVForScope(referencedScope.parentScope)) {
      return compilerCore.createSimpleExpression(
        referencedScope.parentScope.valueAlias + "." + id
      );
    }
    return compilerCore.createSimpleExpression(id);
  } else if (isVForScope(referencedScope)) {
    return compilerCore.createSimpleExpression(referencedScope.valueAlias + "." + id);
  }
  return compilerCore.createSimpleExpression(id);
}
function findReferencedScope(node, scope, findReferenced = true) {
  if (isVIfScope(scope)) {
    return scope;
  } else if (isVForScope(scope)) {
    if (!findReferenced) {
      return scope;
    }
    if (isReferencedByIds(node, scope.locals)) {
      return scope;
    }
    return findReferencedScope(node, scope.parent, findReferenced);
  }
  return scope;
}
function isReferencedByIds(node, knownIds) {
  let referenced = false;
  estreeWalker.walk(node, {
    enter(node2, parent) {
      if (referenced) {
        return this.skip();
      }
      if (!types.isIdentifier(node2)) {
        return;
      }
      if (knownIds.includes(node2.name) && (!parent || types.isReferenced(node2, parent))) {
        referenced = true;
        return this.skip();
      }
    }
  });
  return referenced;
}
function isStaticLiteral(value) {
  return types.isLiteral(value) && !types.isTemplateLiteral(value);
}
function removeAttribute(node, name) {
  const index = node.props.findIndex((prop) => prop.name === name);
  if (index > -1) {
    node.props.splice(index, 1);
  }
}

var BindingComponentTypes = /* @__PURE__ */ ((BindingComponentTypes2) => {
  BindingComponentTypes2["SELF"] = "self";
  BindingComponentTypes2["SETUP"] = "setup";
  BindingComponentTypes2["UNKNOWN"] = "unknown";
  return BindingComponentTypes2;
})(BindingComponentTypes || {});
function isRootScope(scope) {
  return !isVIfScope(scope) && !isVForScope(scope);
}
function isVIfScope(scope) {
  return !!scope.condition || scope.name === "else";
}
function isVForScope(scope) {
  return !!scope.source;
}
function isScopedSlotVFor({ source }) {
  if (source.type !== 8) {
    return false;
  }
  const first = source.children[0];
  return first.type === 4 && first.content.includes(SCOPED_SLOT_IDENTIFIER);
}
function transform(root, options) {
  const context = createTransformContext(root, options);
  findRootNode(root, context);
  traverseNode(root, context);
  root.renderData = createRenderDataExpr(context.scope.properties, context);
  root.helpers = /* @__PURE__ */ new Set([...context.helpers.keys()]);
  root.components = [...context.components];
  root.imports = context.imports;
  root.cached = context.cached;
  return context;
}
function findRootNode(root, context) {
  const children = root.children.filter(
    (node) => node.type === 1 && node.tag !== "template"
  );
  if (children.length === 1) {
    context.rootNode = children[0];
  }
}
function traverseNode(node, context) {
  context.currentNode = node;
  const { nodeTransforms } = context;
  const exitFns = [];
  for (let i2 = 0; i2 < nodeTransforms.length; i2++) {
    const onExit = nodeTransforms[i2](node, context);
    if (onExit) {
      if (shared.isArray(onExit)) {
        exitFns.push(...onExit);
      } else {
        exitFns.push(onExit);
      }
    }
    if (!context.currentNode) {
      return;
    } else {
      node = context.currentNode;
    }
  }
  switch (node.type) {
    case 3:
      break;
    case 5:
      context.helper(compilerCore.TO_DISPLAY_STRING);
      break;
    case 9:
      for (let i2 = 0; i2 < node.branches.length; i2++) {
        traverseNode(node.branches[i2], context);
      }
      break;
    case 10:
    case 11:
    case 1:
    case 0:
      traverseChildren(node, context);
      break;
  }
  context.currentNode = node;
  let i = exitFns.length;
  while (i--) {
    exitFns[i]();
  }
}
function traverseChildren(parent, context) {
  let i = 0;
  const nodeRemoved = () => {
    i--;
  };
  for (; i < parent.children.length; i++) {
    const child = parent.children[i];
    if (shared.isString(child))
      continue;
    context.parent = parent;
    context.childIndex = i;
    context.onNodeRemoved = nodeRemoved;
    traverseNode(child, context);
  }
}
function defaultOnError(error) {
  throw error;
}
function defaultOnWarn(msg) {
  console.warn(`[Vue warn] ${msg.message}`);
}
function createTransformContext(rootNode, {
  root = "",
  filename = "",
  isTS = false,
  inline = false,
  hashId = null,
  scopeId = null,
  filters = [],
  bindingCssVars = [],
  bindingMetadata = shared.EMPTY_OBJ,
  cacheHandlers = false,
  prefixIdentifiers = false,
  skipTransformIdentifier = false,
  renderDataSpread = false,
  nodeTransforms = [],
  directiveTransforms = {},
  miniProgram = {
    class: {
      array: true
    },
    slot: {
      fallbackContent: false,
      dynamicSlotNames: true
    },
    directive: ""
  },
  isBuiltInComponent = shared.NOOP,
  isCustomElement = shared.NOOP,
  expressionPlugins = [],
  onError = defaultOnError,
  onWarn = defaultOnWarn
}) {
  const rootScope = {
    id: new IdentifierGenerator(),
    identifiers: [],
    properties: [],
    parent: null
  };
  function findVIfParentScope() {
    for (let i = scopes.length - 1; i >= 0; i--) {
      const scope = scopes[i];
      if (isVForScope(scope) || isRootScope(scope)) {
        return scope;
      }
    }
    return rootScope;
  }
  function createScope(id, initScope) {
    return shared.extend(
      {
        id,
        properties: [],
        parent: scopes[scopes.length - 1],
        get identifiers() {
          return Object.keys(identifiers);
        }
      },
      initScope
    );
  }
  const vueIds = [];
  const identifiers = /* @__PURE__ */ Object.create(null);
  const scopes = [rootScope];
  const miniProgramComponents = qinjsCliShared.findMiniProgramUsingComponents({
    filename,
    componentsDir: miniProgram.component?.dir,
    inputDir: root
  });
  const context = {
    // options
    // 暂不提供根据文件名生成递归组件
    selfName: "",
    //nameMatch && capitalize(camelize(nameMatch[1])),
    miniProgram,
    isTS,
    inline,
    hashId,
    scopeId,
    filters,
    bindingCssVars,
    bindingMetadata,
    cacheHandlers,
    prefixIdentifiers,
    nodeTransforms,
    directiveTransforms,
    expressionPlugins,
    skipTransformIdentifier,
    renderDataSpread,
    isBuiltInComponent,
    isCustomElement,
    onError,
    onWarn,
    // state
    parent: null,
    childIndex: 0,
    helpers: /* @__PURE__ */ new Map(),
    components: /* @__PURE__ */ new Set(),
    imports: [],
    bindingComponents: /* @__PURE__ */ Object.create(null),
    cached: 0,
    identifiers,
    scope: rootScope,
    scopes: {
      vFor: 0,
      vueId: 0
    },
    get currentScope() {
      return scopes[scopes.length - 1];
    },
    currentNode: rootNode,
    vueIds,
    get currentVueId() {
      return vueIds[vueIds.length - 1];
    },
    inVOnce: false,
    get inVFor() {
      let parent = scopes[scopes.length - 1];
      while (parent) {
        if (isVForScope(parent) && !isScopedSlotVFor(parent)) {
          return true;
        }
        parent = parent.parent;
      }
      return false;
    },
    // methods
    getScopeIndex(scope) {
      return scopes.indexOf(scope);
    },
    popScope() {
      return scopes.pop();
    },
    addVIfScope(initScope) {
      const vIfScope = createScope(
        scopes[scopes.length - 1].id,
        shared.extend(initScope, { parentScope: findVIfParentScope() })
      );
      scopes.push(vIfScope);
      return vIfScope;
    },
    addVForScope(initScope) {
      const vForScope = createScope(
        new IdentifierGenerator(),
        initScope
      );
      scopes.push(vForScope);
      return vForScope;
    },
    helper(name) {
      const count = context.helpers.get(name) || 0;
      context.helpers.set(name, count + 1);
      return name;
    },
    removeHelper(name) {
      const count = context.helpers.get(name);
      if (count) {
        const currentCount = count - 1;
        if (!currentCount) {
          context.helpers.delete(name);
        } else {
          context.helpers.set(name, currentCount);
        }
      }
    },
    helperString(name) {
      return `_${compilerCore.helperNameMap[context.helper(name)]}`;
    },
    replaceNode(node) {
      context.parent.children[context.childIndex] = context.currentNode = node;
    },
    removeNode(node) {
      if (!context.parent) {
        throw new Error(`Cannot remove root node.`);
      }
      const list = context.parent.children;
      const removalIndex = node ? list.indexOf(node) : context.currentNode ? context.childIndex : -1;
      if (removalIndex < 0) {
        throw new Error(`node being removed is not a child of current parent`);
      }
      if (!node || node === context.currentNode) {
        context.currentNode = null;
        context.onNodeRemoved();
      } else {
        if (context.childIndex > removalIndex) {
          context.childIndex--;
          context.onNodeRemoved();
        }
      }
      context.parent.children.splice(removalIndex, 1);
    },
    onNodeRemoved: () => {
    },
    addIdentifiers(exp) {
      if (shared.isString(exp)) {
        addId(exp);
      } else if (exp.identifiers) {
        exp.identifiers.forEach(addId);
      } else if (exp.type === 4) {
        addId(exp.content);
      }
    },
    removeIdentifiers(exp) {
      if (shared.isString(exp)) {
        removeId(exp);
      } else if (exp.identifiers) {
        exp.identifiers.forEach(removeId);
      } else if (exp.type === 4) {
        removeId(exp.content);
      }
    },
    cache(exp, isVNode = false) {
      return createCacheExpression(context.cached++, exp, isVNode);
    },
    isMiniProgramComponent(name) {
      return miniProgramComponents[name];
    },
    rootNode: null
  };
  function addId(id) {
    const { identifiers: identifiers2 } = context;
    if (identifiers2[id] === void 0) {
      identifiers2[id] = 0;
    }
    identifiers2[id]++;
  }
  function removeId(id) {
    context.identifiers[id]--;
  }
  return context;
}
function createCacheExpression(index, value, isVNode = false) {
  return {
    type: 20,
    index,
    value,
    isVNode,
    loc: compilerCore.locStub
  };
}
function createStructuralDirectiveTransform(name, fn) {
  const matches = shared.isString(name) ? (n) => n === name : (n) => name.test(n);
  return (node, context) => {
    if (node.type === 1) {
      const { props } = node;
      const exitFns = [];
      for (let i = 0; i < props.length; i++) {
        const prop = props[i];
        if (prop.type === 7 && matches(prop.name)) {
          props.splice(i, 1);
          i--;
          const onExit = fn(node, prop, context);
          if (onExit)
            exitFns.push(onExit);
        }
      }
      return exitFns;
    }
  };
}
function createRenderDataExpr(properties, context) {
  const objExpr = createObjectExpression(properties);
  if (!hasSpreadElement(objExpr)) {
    return objExpr;
  }
  if (context.filters.length) {
    transformFilterObjectSpreadExpr(objExpr, context);
  }
  if (context.renderDataSpread) {
    return objExpr;
  }
  return transformObjectSpreadExpr(objExpr, context);
}
function hasSpreadElement(expr) {
  return expr.properties.some((prop) => {
    if (types.isSpreadElement(prop)) {
      return true;
    } else {
      const returnStatement = parseReturnStatement(prop);
      if (returnStatement) {
        return hasSpreadElement(returnStatement.argument);
      }
    }
  });
}
const returnObjExprMap = {
  _f: 1,
  // _f(_ctx.items,()=>{return {}})
  _w: 0
  // _w(()=>{return {}})
};
function parseReturnStatement(prop) {
  if (types.isObjectProperty(prop) && types.isCallExpression(prop.value) && types.isIdentifier(prop.value.callee)) {
    const { name } = prop.value.callee;
    if (shared.hasOwn(returnObjExprMap, name)) {
      return prop.value.arguments[returnObjExprMap[name]].body.body[0];
    }
  }
}
function transformObjectPropertyExpr(prop, context) {
  const returnStatement = parseReturnStatement(prop);
  if (returnStatement) {
    const objExpr = returnStatement.argument;
    if (hasSpreadElement(objExpr)) {
      returnStatement.argument = transformObjectSpreadExpr(objExpr, context);
    }
  }
  return prop;
}
function transformObjectSpreadExpr(objExpr, context) {
  const properties = objExpr.properties;
  const args = [];
  let objExprProperties = [];
  properties.forEach((prop) => {
    if (types.isObjectProperty(prop)) {
      objExprProperties.push(transformObjectPropertyExpr(prop, context));
    } else {
      if (objExprProperties.length) {
        args.push(types.objectExpression(objExprProperties));
      }
      args.push(
        transformConditionalExpression(
          prop.argument,
          context
        )
      );
      objExprProperties = [];
    }
  });
  if (objExprProperties.length) {
    args.push(types.objectExpression(objExprProperties));
  }
  if (args.length === 1) {
    return args[0];
  }
  return types.callExpression(types.identifier(context.helperString(EXTEND)), args);
}
function transformConditionalExpression(expr, context) {
  const { consequent, alternate } = expr;
  if (types.isObjectExpression(consequent) && hasSpreadElement(consequent)) {
    expr.consequent = transformObjectSpreadExpr(consequent, context);
  }
  if (types.isObjectExpression(alternate)) {
    if (hasSpreadElement(alternate)) {
      expr.alternate = transformObjectSpreadExpr(alternate, context);
    }
  } else if (types.isConditionalExpression(alternate)) {
    transformConditionalExpression(alternate, context);
  }
  return expr;
}
function transformFilterObjectSpreadExpr(objExpr, context) {
  const properties = objExpr.properties;
  properties.forEach((prop) => {
    if (types.isObjectProperty(prop)) {
      transformFilterObjectPropertyExpr(prop, context);
    } else {
      prop.argument = transformFilterConditionalExpression(
        prop.argument,
        context
      );
    }
  });
}
function transformFilterObjectPropertyExpr(prop, context) {
  const returnStatement = parseReturnStatement(prop);
  if (returnStatement) {
    const objExpr = returnStatement.argument;
    if (hasSpreadElement(objExpr)) {
      transformFilterObjectSpreadExpr(objExpr, context);
    }
  }
}
function transformFilterConditionalExpression(expr, context) {
  const { test, consequent, alternate } = expr;
  if (types.isObjectExpression(consequent) && hasSpreadElement(consequent)) {
    transformFilterObjectSpreadExpr(consequent, context);
  }
  if (types.isObjectExpression(alternate)) {
    if (hasSpreadElement(alternate)) {
      transformFilterObjectSpreadExpr(alternate, context);
    }
  } else if (types.isConditionalExpression(alternate)) {
    expr.alternate = transformFilterConditionalExpression(alternate, context);
  }
  const testCode = genBabelExpr(test);
  if (context.filters.find((filter) => testCode.includes(filter + "."))) {
    const properties = [];
    if (!types.isObjectExpression(consequent) || consequent.properties.length) {
      properties.push(types.spreadElement(consequent));
    }
    if (!types.isObjectExpression(expr.alternate) || expr.alternate.properties.length) {
      properties.push(types.spreadElement(expr.alternate));
    }
    return types.objectExpression(properties);
  }
  return expr;
}

function generate$1(ast, options) {
  const context = createCodegenContext(ast, options);
  const { mode, push, indent, deindent, newline, prefixIdentifiers } = context;
  const helpers = Array.from(ast.helpers);
  const hasHelpers = helpers.length > 0;
  const useWithBlock = !prefixIdentifiers && mode !== "module";
  const isSetupInlined = !!options.inline;
  const preambleContext = isSetupInlined ? createCodegenContext(ast, options) : context;
  if (mode === "module") {
    genModulePreamble(ast, preambleContext, isSetupInlined);
  } else {
    genFunctionPreamble(ast, preambleContext);
  }
  const functionName = `render`;
  const args = ["_ctx", "_cache"];
  if (options.bindingMetadata && !options.inline) {
    args.push("$props", "$setup", "$data", "$options");
  }
  const signature = options.isTS ? args.map((arg) => `${arg}: any`).join(",") : args.join(", ");
  if (isSetupInlined) {
    push(`(${signature}) => {`);
  } else {
    push(`function ${functionName}(${signature}) {`);
  }
  indent();
  if (useWithBlock) {
    push(`with (_ctx) {`);
    indent();
    if (hasHelpers) {
      push(
        `const { ${helpers.map((s) => `${compilerCore.helperNameMap[s]}: _${compilerCore.helperNameMap[s]}`).join(", ")} } = _Vue`
      );
      push(`
`);
      newline();
    }
  }
  push(`return `);
  push(genBabelExpr(ast.renderData, options.generatorOpts));
  if (useWithBlock) {
    deindent();
    push(`}`);
  }
  deindent();
  push(`}`);
  return {
    code: context.code,
    preamble: isSetupInlined ? preambleContext.code : ``,
    // SourceMapGenerator does have toJSON() method but it's not in the types
    map: context.map ? context.map.toJSON() : void 0
  };
}
function createCodegenContext(ast, {
  mode = "function",
  prefixIdentifiers = mode === "module",
  filename = `template.vue.html`,
  scopeId = null,
  runtimeGlobalName = `Vue`,
  runtimeModuleName = `vue`,
  isTS = false,
  sourceMap = false
}) {
  const context = {
    mode,
    prefixIdentifiers,
    filename,
    scopeId,
    runtimeGlobalName,
    runtimeModuleName,
    bindingComponents: ast.bindingComponents,
    isTS,
    source: ast.loc.source,
    code: ``,
    column: 1,
    line: 1,
    offset: 0,
    indentLevel: 0,
    push(code, node) {
      context.code += code;
      if (context.map) {
        if (node) {
          let name;
          if (node.type === 4 && !node.isStatic) {
            const content = node.content.replace(/^_ctx\./, "");
            if (content !== node.content && compilerCore.isSimpleIdentifier(content)) {
              name = content;
            }
          }
          addMapping(node.loc.start, name);
        }
        compilerCore.advancePositionWithMutation(context, code);
        if (node && node.loc !== compilerCore.locStub) {
          addMapping(node.loc.end);
        }
      }
    },
    indent() {
      newline(++context.indentLevel);
    },
    deindent(withoutNewLine = false) {
      if (withoutNewLine) {
        --context.indentLevel;
      } else {
        newline(--context.indentLevel);
      }
    },
    newline() {
      newline(context.indentLevel);
    }
  };
  function newline(n) {
    context.push("\n" + `  `.repeat(n));
  }
  function addMapping(loc, name) {
    context.map.addMapping({
      name,
      source: context.filename || "",
      original: {
        line: loc.line,
        column: loc.column - 1
        // source-map column is 0 based
      },
      generated: {
        line: context.line,
        column: context.column - 1
      }
    });
  }
  return context;
}
function genComponentImports(bindingComponents, { push, newline }) {
  const tags = Object.keys(bindingComponents);
  const importDeclarations = [];
  const components = [];
  tags.forEach((tag) => {
    const { name, type } = bindingComponents[tag];
    if (type === BindingComponentTypes.UNKNOWN) {
      const source = qinjsCliShared.matchEasycom(tag);
      if (source) {
        const easycomName = name.replace("component", "easycom");
        bindingComponents[tag].name = easycomName;
        components.push(easycomName);
        qinjsCliShared.addImportDeclaration(importDeclarations, easycomName, source);
      }
    } else if (type === BindingComponentTypes.SETUP) {
      components.push(name);
    }
  });
  if (tags.length) {
    push(
      `const __BINDING_COMPONENTS__ = '` + JSON.stringify(bindingComponents) + `'`
    );
    const resolveComponents = [];
    const names = [];
    Object.keys(bindingComponents).forEach((id) => {
      const { type, name } = bindingComponents[id];
      if (type === BindingComponentTypes.UNKNOWN) {
        resolveComponents.push(
          `const ${name} = _${compilerCore.helperNameMap[compilerCore.RESOLVE_COMPONENT]}("${id}");`
        );
        names.push(name);
      }
    });
    if (resolveComponents.length) {
      newline();
      push(`if (!Array) {`);
      resolveComponents.forEach((code) => {
        push(code);
      });
      push(`(${names.join("+")})()`);
      push(`}`);
    }
    newline();
    importDeclarations.forEach((str) => push(str));
    if (importDeclarations.length) {
      newline();
    }
    if (components.length) {
      push(`if (!Math) {`);
      push(` (${components.map((name) => name).join("+")})() `);
      push(`}`);
      newline();
    }
  }
}
function genFunctionPreamble(ast, context) {
  const {
    prefixIdentifiers,
    push,
    newline,
    runtimeGlobalName,
    bindingComponents
  } = context;
  const VueBinding = runtimeGlobalName;
  const aliasHelper = (s) => `${compilerCore.helperNameMap[s]}: _${compilerCore.helperNameMap[s]}`;
  const helpers = Array.from(ast.helpers);
  if (helpers.length > 0) {
    if (prefixIdentifiers) {
      push(
        `const { ${helpers.map(aliasHelper).join(", ")} } = ${VueBinding}
`
      );
    } else {
      push(`const _Vue = ${VueBinding}
`);
    }
  }
  genComponentImports(bindingComponents, context);
  newline();
  push(`return `);
}
function genModulePreamble(ast, context, inline) {
  const { push, newline, runtimeModuleName, bindingComponents } = context;
  const helpers = Array.from(ast.helpers);
  if (helpers.length) {
    push(
      `import { ${helpers.map((s) => `${compilerCore.helperNameMap[s]} as _${compilerCore.helperNameMap[s]}`).join(", ")} } from ${JSON.stringify(runtimeModuleName)}
`
    );
  }
  if (ast.imports.length) {
    genImports(ast.imports, context);
  }
  genComponentImports(bindingComponents, context);
  newline();
  if (!inline) {
    push(`export `);
  }
}
function genImports(importsOptions, { push, newline }) {
  if (!importsOptions.length) {
    return;
  }
  importsOptions.forEach((imports) => {
    push(`import `);
    push(genExpr(imports.exp));
    push(` from '${imports.path}'`);
    newline();
  });
}
function createGenNodeContext() {
  const context = {
    code: "",
    helper(key) {
      return `_${compilerCore.helperNameMap[key]}`;
    },
    push(code) {
      context.code += code;
    }
  };
  return context;
}
function genBabelExpr(expr, opts = {}) {
  if (!shared.hasOwn(opts, "jsescOption")) {
    opts.jsescOption = {};
  }
  opts.jsescOption.quotes = "single";
  return (babelGenerate__default.default || babelGenerate__default)(expr, opts).code;
}
function genExpr(node, context) {
  return genNode$1(node, context).code;
}
function genNode$1(node, context) {
  if (!context) {
    context = createGenNodeContext();
  }
  if (shared.isString(node)) {
    context.push(node);
    return context;
  }
  if (shared.isSymbol(node)) {
    context.push(context.helper(node));
    return context;
  }
  switch (node.type) {
    case 2:
      genText$1(node, context);
      break;
    case 4:
      genExpression$1(node, context);
      break;
    case 5:
      genInterpolation(node, context);
      break;
    case 8:
      genCompoundExpression(node, context);
      break;
  }
  return context;
}
function genText$1(node, context) {
  context.push(JSON.stringify(node.content), node);
}
function genExpression$1(node, context) {
  const { content, isStatic } = node;
  context.push(isStatic ? JSON.stringify(content) : content, node);
}
function genInterpolation(node, context) {
  const { push, helper } = context;
  push(`${helper(compilerCore.TO_DISPLAY_STRING)}(`);
  genExpr(node.content, context);
  push(`)`);
}
function genCompoundExpression(node, context) {
  for (let i = 0; i < node.children.length; i++) {
    const child = node.children[i];
    if (shared.isString(child)) {
      context.push(child);
    } else {
      genExpr(child, context);
    }
  }
}

const isLiteralWhitelisted = /* @__PURE__ */ shared.makeMap("true,false,null,this");
const transformExpression = (node, context) => {
  if (node.type === 5) {
    node.content = processExpression(
      node.content,
      context
    );
  } else if (node.type === 1) {
    for (let i = 0; i < node.props.length; i++) {
      const dir = node.props[i];
      if (dir.type === 7 && dir.name !== "for") {
        const exp = dir.exp;
        const arg = dir.arg;
        if (exp && exp.type === 4 && !(dir.name === "on" && arg)) {
          dir.exp = processExpression(
            exp,
            context,
            // slot args must be processed as function params
            dir.name === "slot"
          );
        }
        if (arg && arg.type === 4 && !arg.isStatic) {
          dir.arg = processExpression(arg, context);
        }
      }
    }
  }
};
function processExpression(node, context, asParams = false, asRawStatements = false, localVars = Object.create(context.identifiers)) {
  if (!context.prefixIdentifiers || !node.content.trim()) {
    return node;
  }
  const { inline, bindingMetadata } = context;
  const rewriteIdentifier = (raw, parent, id) => {
    const type = shared.hasOwn(bindingMetadata, raw) && bindingMetadata[raw];
    if (inline) {
      const isAssignmentLVal = parent && parent.type === "AssignmentExpression" && parent.left === id;
      const isUpdateArg = parent && parent.type === "UpdateExpression" && parent.argument === id;
      const isDestructureAssignment = parent && compilerCore.isInDestructureAssignment(parent, parentStack);
      if (type === "setup-const" || type === "setup-reactive-const" || localVars[raw]) {
        return raw;
      } else if (type === "setup-ref") {
        return `${raw}.value`;
      } else if (type === "setup-maybe-ref") {
        return isAssignmentLVal || isUpdateArg || isDestructureAssignment ? `${raw}.value` : `${context.helperString(compilerCore.UNREF)}(${raw})`;
      } else if (type === "setup-let") {
        if (isAssignmentLVal) {
          const { right: rVal, operator } = parent;
          const rExp = rawExp.slice(rVal.start - 1, rVal.end - 1);
          const rExpString = stringifyExpression(
            processExpression(
              compilerCore.createSimpleExpression(rExp, false),
              context,
              false,
              false,
              knownIds
            )
          );
          return `${context.helperString(compilerCore.IS_REF)}(${raw})${context.isTS ? ` //@ts-ignore
` : ``} ? ${raw}.value ${operator} ${rExpString} : ${raw}`;
        } else if (isUpdateArg) {
          id.start = parent.start;
          id.end = parent.end;
          const { prefix: isPrefix, operator } = parent;
          const prefix = isPrefix ? operator : ``;
          const postfix = isPrefix ? `` : operator;
          return `${context.helperString(compilerCore.IS_REF)}(${raw})${context.isTS ? ` //@ts-ignore
` : ``} ? ${prefix}${raw}.value${postfix} : ${prefix}${raw}${postfix}`;
        } else if (isDestructureAssignment) {
          return raw;
        } else {
          return `${context.helperString(compilerCore.UNREF)}(${raw})`;
        }
      } else if (type === "props") {
        return shared.genPropsAccessExp(raw);
      } else if (type === "props-aliased") {
        return shared.genPropsAccessExp(bindingMetadata.__propsAliases[raw]);
      }
    } else {
      if (type && type.startsWith("setup")) {
        return `$setup.${raw}`;
      } else if (type === "props-aliased") {
        return `$props['${bindingMetadata.__propsAliases[raw]}']`;
      } else if (type) {
        return `$${type}.${raw}`;
      }
    }
    return `_ctx.${raw}`;
  };
  const rawExp = node.content;
  const bailConstant = rawExp.indexOf(`(`) > -1 || rawExp.indexOf(".") > 0;
  if (compilerCore.isSimpleIdentifier(rawExp)) {
    const isScopeVarReference = context.identifiers[rawExp];
    const isAllowedGlobal = shared.isGloballyWhitelisted(rawExp);
    const isLiteral = isLiteralWhitelisted(rawExp);
    const isFilter = context.filters.includes(rawExp);
    const isBuiltIn = isBuiltInIdentifier(rawExp);
    if (!asParams && !isScopeVarReference && !isAllowedGlobal && !isLiteral && !isFilter && !isBuiltIn) {
      if (bindingMetadata[node.content] === "setup-const") {
        node.constType = 1;
      }
      node.content = rewriteIdentifier(rawExp);
    } else if (!isScopeVarReference) {
      if (isLiteral) {
        node.constType = 3;
      } else {
        node.constType = 2;
      }
    }
    return node;
  }
  let ast;
  const source = asRawStatements ? ` ${rawExp} ` : `(${rawExp})${asParams ? `=>{}` : ``}`;
  try {
    ast = parser.parse(source, {
      plugins: context.expressionPlugins
    }).program;
  } catch (e) {
    context.onError(
      compilerCore.createCompilerError(
        45,
        node.loc,
        void 0,
        "\n" + source + "\n" + e.message
      )
    );
    return node;
  }
  const ids = [];
  const parentStack = [];
  const knownIds = Object.create(context.identifiers);
  context.filters.forEach((name) => {
    knownIds[name] = 1;
  });
  compilerCore.walkIdentifiers(
    ast,
    (node2, parent, _, isReferenced, isLocal) => {
      if (compilerCore.isStaticPropertyKey(node2, parent)) {
        return;
      }
      const needPrefix = isReferenced && canPrefix(node2);
      if (needPrefix && !isLocal) {
        if (compilerCore.isStaticProperty(parent) && parent.shorthand) {
          node2.prefix = `${node2.name}: `;
        }
        node2.name = rewriteIdentifier(node2.name, parent, node2);
        ids.push(node2);
      } else {
        if (!(needPrefix && isLocal) && !bailConstant) {
          node2.isConstant = true;
        }
        ids.push(node2);
      }
    },
    true,
    // invoke on ALL identifiers
    parentStack,
    knownIds
  );
  const children = [];
  ids.sort((a, b) => a.start - b.start);
  ids.forEach((id, i) => {
    const start = id.start - 1;
    const end = id.end - 1;
    const last = ids[i - 1];
    const leadingText = rawExp.slice(last ? last.end - 1 : 0, start);
    if (leadingText.length || id.prefix) {
      children.push(leadingText + (id.prefix || ``));
    }
    const source2 = rawExp.slice(start, end);
    children.push(
      compilerCore.createSimpleExpression(
        id.name,
        false,
        {
          source: source2,
          start: compilerCore.advancePositionWithClone(node.loc.start, source2, start),
          end: compilerCore.advancePositionWithClone(node.loc.start, source2, end)
        },
        id.isConstant ? 3 : 0
      )
    );
    if (i === ids.length - 1 && end < rawExp.length) {
      children.push(rawExp.slice(end));
    }
  });
  let ret;
  if (children.length) {
    ret = compilerCore.createCompoundExpression(children, node.loc);
  } else {
    ret = node;
    ret.constType = bailConstant ? 0 : 3;
  }
  ret.identifiers = Object.keys(knownIds);
  return ret;
}
function canPrefix(id) {
  if (shared.isGloballyWhitelisted(id.name)) {
    return false;
  }
  if (id.name === "require") {
    return false;
  }
  return true;
}
function stringifyExpression(exp) {
  if (shared.isString(exp)) {
    return exp;
  } else if (exp.type === 4) {
    return exp.content;
  } else {
    return exp.children.map(stringifyExpression).join("");
  }
}
const builtInIdentifiers = ["__l"];
function isBuiltInIdentifier(id) {
  if (!shared.isString(id)) {
    if (id.type !== 4) {
      return false;
    }
    id = id.content;
  }
  return builtInIdentifiers.includes(id);
}

function isClassBinding({ arg }) {
  return arg && arg.type === 4 && arg.content === "class";
}
function findStaticClassIndex(props) {
  return props.findIndex((prop) => prop.name === "class");
}
function rewriteClass(index, classBindingProp, props, virtualHost, context) {
  const expr = classBindingProp.exp ? parseExpr(classBindingProp.exp, context) : void 0;
  let classBindingExpr;
  if (expr) {
    classBindingExpr = expr;
    if (types.isObjectExpression(expr)) {
      classBindingExpr = createClassBindingByObjectExpression(
        rewriteClassObjectExpression(expr, classBindingProp.loc, context)
      );
    } else if (types.isArrayExpression(expr)) {
      classBindingExpr = createClassBindingByArrayExpression(
        rewriteClassArrayExpression(expr, context)
      );
    } else {
      classBindingExpr = parseExpr(
        rewriteClassExpression(classBindingProp.exp, context).content,
        context
      );
    }
  } else if (virtualHost) {
    classBindingExpr = types.arrayExpression([]);
  } else {
    return;
  }
  const staticClassPropIndex = findStaticClassIndex(props);
  if (staticClassPropIndex > -1) {
    const staticClass = props[staticClassPropIndex].value.content;
    if (staticClass) {
      if (!types.isArrayExpression(classBindingExpr)) {
        classBindingExpr = types.arrayExpression([classBindingExpr]);
      }
      const staticClassLiterals = parseStaticClass(staticClass);
      if (index > staticClassPropIndex) {
        classBindingExpr.elements.unshift(...staticClassLiterals);
      } else {
        classBindingExpr.elements.push(...staticClassLiterals);
      }
    }
  }
  if (virtualHost) {
    if (!types.isArrayExpression(classBindingExpr)) {
      classBindingExpr = types.arrayExpression([classBindingExpr]);
    }
    classBindingExpr.elements.push(types.identifier(VIRTUAL_HOST_CLASS));
  }
  if (!context.miniProgram.class.array) {
    classBindingExpr = parseClassBindingArrayExpr(classBindingExpr);
  }
  classBindingProp.exp = compilerCore.createSimpleExpression(genBabelExpr(classBindingExpr));
}
function createVirtualHostClass(props, context) {
  const classBindingProp = qinjsCliShared.createBindDirectiveNode("class", "");
  delete classBindingProp.exp;
  rewriteClass(0, classBindingProp, props, true, context);
  return classBindingProp;
}
function parseClassBindingArrayExpr(classBindingExpr) {
  if (!types.isArrayExpression(classBindingExpr)) {
    return classBindingExpr;
  }
  let binaryExpr;
  classBindingExpr.elements.forEach((expr) => {
    if (types.isArrayExpression(expr)) {
      expr = parseClassBindingArrayExpr(expr);
    }
    if (!binaryExpr) {
      binaryExpr = types.parenthesizedExpression(expr);
    } else {
      binaryExpr = types.binaryExpression(
        "+",
        types.binaryExpression("+", binaryExpr, types.stringLiteral(" ")),
        expr
      );
    }
  });
  return binaryExpr;
}
function parseStaticClass(staticClass) {
  return staticClass.split(" ").map((clazz) => types.stringLiteral(clazz));
}
function rewriteClassExpression(expr, context) {
  return rewriteExpression(
    compilerCore.createCompoundExpression([
      context.helperString(NORMALIZE_CLASS) + "(",
      expr,
      ")"
    ]),
    context
  );
}
function rewriteClassArrayExpression(expr, context) {
  expr.elements.forEach((prop, index) => {
    if (!types.isStringLiteral(prop)) {
      const code = genBabelExpr(
        types.arrayExpression([types.isSpreadElement(prop) ? prop.argument : prop])
      );
      expr.elements[index] = types.identifier(
        rewriteClassExpression(
          compilerCore.createSimpleExpression(code.slice(1, -1), false),
          context
        ).content
      );
    }
  });
  return expr;
}
function rewriteClassObjectExpression(expr, loc, context) {
  expr.properties.forEach((prop, index) => {
    if (types.isSpreadElement(prop)) {
      const newExpr = rewriteSpreadElement(NORMALIZE_CLASS, prop, loc, context);
      if (newExpr) {
        expr.properties[index] = types.objectProperty(
          newExpr,
          types.booleanLiteral(true),
          true
        );
      }
    } else if (types.isObjectProperty(prop)) {
      const { key, value, computed } = prop;
      if (computed) {
        prop.key = parseExprWithRewrite(
          genBabelExpr(key),
          loc,
          context,
          key
        );
      }
      if (isStaticLiteral(value)) {
        return;
      } else {
        const newExpr = parseExprWithRewriteClass(
          genBabelExpr(value),
          loc,
          context,
          value
        );
        if (newExpr) {
          prop.value = newExpr;
        }
      }
    }
  });
  return expr;
}
function createClassBindingByArrayExpression(expr) {
  const elements = [];
  expr.elements.forEach((prop) => {
    if (types.isStringLiteral(prop) || types.isIdentifier(prop)) {
      elements.push(prop);
    }
  });
  return types.arrayExpression(elements);
}
function createClassBindingByObjectExpression(expr) {
  const elements = [];
  expr.properties.forEach((prop) => {
    if (types.isObjectProperty(prop)) {
      const { value } = prop;
      if (isUndefined(value) || types.isPrivateName(prop.key)) {
        return;
      }
      if (types.isLiteral(value)) {
        if (isTrueExpr(value)) {
          elements.push(
            prop.computed ? prop.key : parseStringLiteral(prop.key)
          );
        }
        return;
      }
      elements.push(
        types.logicalExpression(
          "&&",
          value,
          prop.computed ? prop.key : parseStringLiteral(prop.key)
        )
      );
    }
  });
  return types.arrayExpression(elements);
}

function isStyleBinding({ arg, exp }) {
  return arg && arg.type === 4 && arg.content === "style";
}
function findStaticStyleIndex(props) {
  return props.findIndex((prop) => prop.name === "style");
}
function rewriteStyle(index, styleBindingProp, props, virtualHost, context) {
  const expr = styleBindingProp.exp ? parseExpr(styleBindingProp.exp, context) : void 0;
  let styleBidingExpr = expr;
  if (expr) {
    if (types.isObjectExpression(expr)) {
      styleBidingExpr = createStyleBindingByObjectExpression(
        rewriteStyleObjectExpression(expr, styleBindingProp.loc, context)
      );
    } else if (types.isArrayExpression(expr)) {
      styleBidingExpr = createStyleBindingByArrayExpression(
        rewriteStyleArrayExpression(expr, context)
      );
    } else {
      styleBidingExpr = parseExpr(
        rewriteStyleExpression(styleBindingProp.exp, context).content,
        context
      );
    }
    if (!styleBidingExpr) {
      return;
    }
  } else if (!virtualHost) {
    return;
  }
  const staticStylePropIndex = findStaticStyleIndex(props);
  if (staticStylePropIndex > -1) {
    const staticStyle = props[staticStylePropIndex].value.content;
    if (staticStyle.trim()) {
      if (styleBidingExpr) {
        if (index > staticStylePropIndex) {
          styleBidingExpr = types.binaryExpression(
            "+",
            addSemicolon(types.stringLiteral(staticStyle)),
            styleBidingExpr
          );
        } else {
          styleBidingExpr = types.binaryExpression(
            "+",
            addSemicolon(styleBidingExpr),
            types.stringLiteral(staticStyle)
          );
        }
      } else {
        styleBidingExpr = types.stringLiteral(staticStyle);
      }
    }
  }
  if (virtualHost) {
    styleBidingExpr = styleBidingExpr ? types.binaryExpression(
      "+",
      addSemicolon(styleBidingExpr),
      types.identifier(VIRTUAL_HOST_STYLE)
    ) : types.identifier(VIRTUAL_HOST_STYLE);
  }
  styleBindingProp.exp = compilerCore.createSimpleExpression(genBabelExpr(styleBidingExpr));
}
function createVirtualHostStyle(props, context) {
  const styleBindingProp = qinjsCliShared.createBindDirectiveNode("style", "");
  delete styleBindingProp.exp;
  rewriteStyle(0, styleBindingProp, props, true, context);
  return styleBindingProp;
}
function rewriteStyleExpression(expr, context) {
  return rewriteExpression(
    compilerCore.createCompoundExpression([
      context.helperString(STRINGIFY_STYLE) + "(",
      expr,
      ")"
    ]),
    context
  );
}
function rewriteStyleArrayExpression(expr, context) {
  expr.elements.forEach((prop, index) => {
    if (!types.isStringLiteral(prop)) {
      const code = genBabelExpr(
        types.arrayExpression([types.isSpreadElement(prop) ? prop.argument : prop])
      );
      expr.elements[index] = types.identifier(
        rewriteStyleExpression(
          compilerCore.createSimpleExpression(code.slice(1, -1), false),
          context
        ).content
      );
    }
  });
  return expr;
}
function rewriteStyleObjectExpression(expr, loc, context) {
  expr.properties.forEach((prop, index) => {
    if (types.isSpreadElement(prop)) {
      const newExpr = rewriteSpreadElement(STRINGIFY_STYLE, prop, loc, context);
      if (newExpr) {
        prop.argument = newExpr;
      }
    } else if (types.isObjectProperty(prop)) {
      const { key, value, computed } = prop;
      if (!types.isPrivateName(key)) {
        if (computed) {
          const newExpr = rewirteWithHelper(HYPHENATE, key, loc, context);
          if (newExpr) {
            prop.key = newExpr;
          }
        } else {
          prop.key = parseStringLiteral(key);
          prop.key.value = shared.hyphenate(prop.key.value) + ":";
        }
        if (isStaticLiteral(value)) {
          return;
        } else {
          const newExpr = parseExprWithRewrite(
            genBabelExpr(value),
            loc,
            context,
            value
          );
          if (newExpr) {
            prop.value = newExpr;
          }
        }
      }
    }
  });
  return expr;
}
function addSemicolon(expr) {
  return createBinaryExpression(expr, types.stringLiteral(";"));
}
function createBinaryExpression(left, right) {
  return types.binaryExpression("+", left, right);
}
function createStyleBindingByArrayExpression(expr) {
  let result;
  function concat(expr2) {
    if (!result) {
      result = expr2;
    } else {
      result = createBinaryExpression(addSemicolon(result), expr2);
    }
  }
  expr.elements.forEach((prop) => {
    if (types.isStringLiteral(prop) || types.isIdentifier(prop)) {
      concat(prop);
    }
  });
  return result;
}
function createStyleBindingByObjectExpression(expr) {
  let result;
  function concat(expr2) {
    if (!result) {
      result = expr2;
    } else {
      result = createBinaryExpression(addSemicolon(result), expr2);
    }
  }
  expr.properties.forEach((prop) => {
    if (types.isSpreadElement(prop)) {
      concat(prop.argument);
    } else if (types.isObjectProperty(prop)) {
      const { key, value } = prop;
      if (!types.isPrivateName(key)) {
        const expr2 = createBinaryExpression(
          types.isStringLiteral(key) ? key : createBinaryExpression(key, types.stringLiteral(":")),
          value
        );
        concat(expr2);
      }
    }
  });
  return result;
}

var MPErrorCodes = /* @__PURE__ */ ((MPErrorCodes2) => {
  MPErrorCodes2[MPErrorCodes2["X_V_ON_NO_ARGUMENT"] = 0] = "X_V_ON_NO_ARGUMENT";
  MPErrorCodes2[MPErrorCodes2["X_V_ON_DYNAMIC_EVENT"] = 1] = "X_V_ON_DYNAMIC_EVENT";
  MPErrorCodes2[MPErrorCodes2["X_V_BIND_NO_ARGUMENT"] = 2] = "X_V_BIND_NO_ARGUMENT";
  MPErrorCodes2[MPErrorCodes2["X_V_BIND_DYNAMIC_ARGUMENT"] = 3] = "X_V_BIND_DYNAMIC_ARGUMENT";
  MPErrorCodes2[MPErrorCodes2["X_V_BIND_MODIFIER_PROP"] = 4] = "X_V_BIND_MODIFIER_PROP";
  MPErrorCodes2[MPErrorCodes2["X_V_BIND_MODIFIER_ATTR"] = 5] = "X_V_BIND_MODIFIER_ATTR";
  MPErrorCodes2[MPErrorCodes2["X_V_IS_NOT_SUPPORTED"] = 6] = "X_V_IS_NOT_SUPPORTED";
  MPErrorCodes2[MPErrorCodes2["X_NOT_SUPPORTED"] = 7] = "X_NOT_SUPPORTED";
  MPErrorCodes2[MPErrorCodes2["X_DYNAMIC_COMPONENT_NOT_SUPPORTED"] = 8] = "X_DYNAMIC_COMPONENT_NOT_SUPPORTED";
  return MPErrorCodes2;
})(MPErrorCodes || {});
const MPErrorMessages = {
  [0 /* X_V_ON_NO_ARGUMENT */]: 'v-on="" is not supported',
  [1 /* X_V_ON_DYNAMIC_EVENT */]: 'v-on:[event]="" is not supported.',
  [2 /* X_V_BIND_NO_ARGUMENT */]: 'v-bind="" is not supported.',
  [3 /* X_V_BIND_DYNAMIC_ARGUMENT */]: 'v-bind:[name]="" is not supported.',
  [4 /* X_V_BIND_MODIFIER_PROP */]: "v-bind .prop is not supported",
  [5 /* X_V_BIND_MODIFIER_ATTR */]: "v-bind .attr is not supported",
  [8 /* X_DYNAMIC_COMPONENT_NOT_SUPPORTED */]: '<component is=""/> is not supported',
  [7 /* X_NOT_SUPPORTED */]: "not supported: ",
  [6 /* X_V_IS_NOT_SUPPORTED */]: "v-is not supported"
};
function createMPCompilerError(code, loc, additionalMessage) {
  return compilerCore.createCompilerError(
    code,
    loc,
    MPErrorMessages,
    additionalMessage
  );
}

const transformSlot = (node, context) => {
  if (!qinjsCliShared.isUserComponent(node, context)) {
    return;
  }
  const { tag, children } = node;
  const slots = /* @__PURE__ */ new Set();
  const onComponentSlot = compilerCore.findDir(node, "slot", true);
  const implicitDefaultChildren = [];
  const isMiniProgramComponent = context.isMiniProgramComponent(tag);
  for (let i = 0; i < children.length; i++) {
    const slotElement = children[i];
    let slotDir;
    if (!compilerCore.isTemplateNode(slotElement) || !(slotDir = compilerCore.findDir(slotElement, "slot", true))) {
      if (slotElement.type !== 3) {
        implicitDefaultChildren.push(slotElement);
      }
      continue;
    }
    if (onComponentSlot) {
      context.onError(
        compilerCore.createCompilerError(
          37,
          slotDir.loc
        )
      );
      break;
    }
    if (!slotDir.arg) {
      slotDir.arg = compilerCore.createSimpleExpression("default", true);
    }
    const slotName = transformTemplateSlotElement(
      slotDir,
      slotElement,
      node,
      context
    );
    if (isMiniProgramComponent) {
      if (slotName === "default" && slotElement.children.length === 1) {
        children.splice(i, 1, slotElement.children[0]);
      }
      continue;
    }
    if (slotName) {
      slots.add(slotName);
    }
  }
  if (isMiniProgramComponent) {
    return;
  }
  if (implicitDefaultChildren.length) {
    slots.add(qinjsShared.SLOT_DEFAULT_NAME);
  }
  if (onComponentSlot) {
    slots.add(qinjsShared.SLOT_DEFAULT_NAME);
    const templateNode = createTemplateNode(
      onComponentSlot,
      implicitDefaultChildren
    );
    transformTemplateSlotElement(onComponentSlot, templateNode, node, context);
    node.children = [templateNode];
  }
  if (slots.size && !context.miniProgram.slot.$slots) {
    const slotsArr = [...slots];
    const hasDynamic = slotsArr.find((name) => !shared.isString(name));
    let value;
    if (hasDynamic) {
      const children2 = [];
      const len = slotsArr.length - 1;
      slotsArr.forEach((name, index) => {
        if (shared.isString(name)) {
          children2.push(`'${qinjsShared.dynamicSlotName(name)}'`);
        } else {
          children2.push(name);
        }
        if (index < len) {
          children2.push(",");
        }
      });
      value = compilerCore.createCompoundExpression([
        context.helperString(DYNAMIC_SLOT) + "([",
        ...children2,
        "])"
      ]);
    } else {
      value = `[${slotsArr.map((name) => `'${qinjsShared.dynamicSlotName(name)}'`).join(",")}]`;
    }
    node.props.unshift(qinjsCliShared.createBindDirectiveNode(ATTR_VUE_SLOTS, value));
  }
};
function rewriteVSlot(dir, context) {
  dir.arg = rewriteExpression(
    compilerCore.createCompoundExpression([
      context.helperString(DYNAMIC_SLOT) + "(",
      dir.arg,
      ")"
    ]),
    context
  );
}
function transformTemplateSlotElement(slotDir, slotTemplate, slotComponent, context) {
  const slotName = findSlotName(slotDir);
  if (!slotName) {
    return;
  }
  const { exp } = slotDir;
  if (!exp) {
    return slotName;
  }
  if (exp.type === 4 && !exp.content.trim()) {
    return slotName;
  }
  slotTemplate.children = [
    createVForTemplate(
      slotTemplate,
      { name: slotName, value: genExpr(exp), slotComponent },
      context
    )
  ];
  if (context.miniProgram.slot.dynamicSlotNames) {
    const index = slotTemplate.props.indexOf(slotDir);
    if (index > -1) {
      slotTemplate.props.splice(index, 1);
    }
  }
  slotDir.exp = void 0;
  return slotName;
}
function createTemplateNode(slotDir, children) {
  return {
    type: 1,
    tag: "template",
    tagType: 3,
    loc: compilerCore.locStub,
    isSelfClosing: false,
    codegenNode: void 0,
    ns: 0,
    props: [slotDir],
    children
  };
}
function findSlotName(slotDir) {
  if (!slotDir.arg) {
    return qinjsShared.SLOT_DEFAULT_NAME;
  }
  if (compilerCore.isStaticExp(slotDir.arg)) {
    return slotDir.arg.content;
  }
  return slotDir.arg;
}
function findCurrentVForValueAlias(context) {
  let scope = context.currentScope;
  while (scope) {
    if (isVForScope(scope)) {
      return scope.valueAlias;
    }
    scope = scope.parent;
  }
  return "";
}
function createVForTemplate(slotElement, {
  name,
  value,
  slotComponent
}, context) {
  const slotName = "s" + context.scopes.vFor;
  const keyProp = qinjsCliShared.createBindDirectiveNode(
    "key",
    "i" + context.scopes.vFor
  );
  const source = shared.isString(name) ? `'${name}'` : genExpr(name);
  const vForProp = {
    type: 7,
    name: "for",
    loc: compilerCore.locStub,
    modifiers: [],
    arg: void 0,
    exp: compilerCore.createSimpleExpression(
      `(${value}, ${slotName}) in ${SCOPED_SLOT_IDENTIFIER}(${source}, ${findCurrentVForValueAlias(context) || `''`})`
    )
  };
  const props = [vForProp, keyProp];
  if (context.miniProgram.slot.dynamicSlotNames) {
    props.push(qinjsCliShared.createBindDirectiveNode("slot", slotName));
  }
  return {
    loc: slotElement.loc,
    ns: 0,
    tag: "template",
    type: 1,
    tagType: 3,
    props,
    isSelfClosing: false,
    codegenNode: void 0,
    children: slotElement.children,
    slotComponent
  };
}
const slotNameRE = /\('(.*)',/;
function findCurrentSlotName(source) {
  return types.stringLiteral(
    qinjsShared.dynamicSlotName(
      source.children[1].match(
        slotNameRE
      )[1]
    )
  );
}
function createPathBinaryExpr(scope, computed = true) {
  return types.binaryExpression(
    "+",
    types.binaryExpression(
      "+",
      types.stringLiteral(parseVForPath(scope.sourceAlias) + (computed ? "[" : ".")),
      types.identifier(scope.indexAlias)
    ),
    types.stringLiteral(computed ? "]." : ".")
  );
}
function findCurrentPath(id, scope) {
  let parent = scope.parent;
  let binaryExpr = null;
  while (parent) {
    if (isVForScope(parent)) {
      if (!binaryExpr) {
        binaryExpr = createPathBinaryExpr(parent);
      } else {
        binaryExpr = types.binaryExpression(
          "+",
          createPathBinaryExpr(parent),
          binaryExpr
        );
      }
    }
    parent = parent.parent;
  }
  return binaryExpr && types.binaryExpression("+", binaryExpr, types.stringLiteral(id)) || types.stringLiteral(id);
}
function findCurrentVueIdExpr(node, context) {
  if (!node) {
    return types.stringLiteral("");
  }
  const vueIdProp = compilerCore.findProp(node, ATTR_VUE_ID);
  if (vueIdProp.type === 6) {
    return types.stringLiteral(vueIdProp.value.content);
  }
  return parseExpr(genExpr(vueIdProp.exp), context) || types.stringLiteral("");
}
function parseVForPath(id) {
  return id.includes(".") ? id.split(".")[1] : id;
}
function createVSlotCallExpression(slotComponent, vForScope, context) {
  const { source, sourceAlias } = vForScope;
  const id = parseVForPath(sourceAlias);
  return types.callExpression(types.identifier(context.helperString(WITH_SCOPED_SLOT)), [
    createVForArrowFunctionExpression(vForScope),
    types.objectExpression([
      // 插槽名称，数据更新 path，vueId
      types.objectProperty(types.identifier("name"), findCurrentSlotName(source)),
      // 暂不生成 path
      types.objectProperty(types.identifier("path"), findCurrentPath(id, vForScope)),
      types.objectProperty(
        types.identifier("vueId"),
        findCurrentVueIdExpr(slotComponent, context)
      )
    ])
  ]);
}

function isForElementNode(node) {
  return !!node.vFor;
}
const transformFor = createStructuralDirectiveTransform(
  "for",
  (node, dir, context) => {
    if (!dir.exp) {
      context.onError(
        compilerCore.createCompilerError(31, dir.loc)
      );
      return;
    }
    const parseResult = parseForExpression(
      dir.exp,
      context
    );
    if (!parseResult) {
      context.onError(
        compilerCore.createCompilerError(
          32,
          dir.loc
        )
      );
      return;
    }
    parseResult.tagType = node.tagType;
    const { addIdentifiers, removeIdentifiers } = context;
    const { source, value, key, index } = parseResult;
    if (context.prefixIdentifiers) {
      addIdentifiers(value);
      addIdentifiers(key);
      addIdentifiers(index);
    }
    const { currentScope: parentScope, scopes, popScope } = context;
    const sourceExpr = parseExpr(source, context);
    const valueCode = genExpr(value);
    const valueExpr = parseParam(valueCode, context, value);
    const valueAlias = parseAlias(valueExpr, valueCode, "v" + scopes.vFor);
    const keyCode = genExpr(key);
    const keyExpr = parseParam(keyCode, context, key);
    const keyAlias = parseAlias(keyExpr, keyCode, "k" + scopes.vFor);
    const indexCode = genExpr(index);
    const indexExpr = parseParam(indexCode, context, index);
    const indexAlias = parseAlias(indexExpr, indexCode, "i" + scopes.vFor);
    const cloneSourceExpr = types.cloneNode(sourceExpr, false);
    const sourceAliasReferencedScope = findReferencedScope(
      cloneSourceExpr,
      context.currentScope,
      // vFor 嵌套时，始终保持嵌套关系，issues/3263
      false
    );
    const vIfReferencedScope = findVIfReferencedScope(
      node,
      context.currentScope,
      context
    );
    const referencedScope = vIfReferencedScope && context.getScopeIndex(vIfReferencedScope) > context.getScopeIndex(sourceAliasReferencedScope) ? vIfReferencedScope : sourceAliasReferencedScope;
    const sourceAlias = rewriteExpression(
      source,
      context,
      cloneSourceExpr,
      parentScope,
      // 强制 rewrite，因为即使是字符串，数字，也要走 vFor 函数
      {
        property: true,
        ignoreLiteral: true,
        referencedScope
      }
    ).content;
    const sourceCode = `{{${sourceAlias}}}`;
    const vForData = {
      source,
      sourceExpr,
      sourceAlias,
      sourceCode,
      value,
      valueCode,
      valueExpr,
      valueAlias,
      key,
      keyCode,
      keyExpr,
      keyAlias,
      index,
      indexCode,
      indexExpr,
      indexAlias,
      node
    };
    const vForScope = context.addVForScope({
      ...vForData,
      locals: findVForLocals(parseResult)
    });
    const vFor = {
      ...vForData
    };
    const isScopedSlot = isScopedSlotVFor(vForScope);
    node.vFor = vFor;
    scopes.vFor++;
    return () => {
      scopes.vFor--;
      if (compilerCore.isTemplateNode(node)) {
        node.children.some((c) => {
          if (c.type === 1 && !isForElementNode(c)) {
            const key2 = compilerCore.findProp(c, "key");
            if (key2) {
              context.onError(
                compilerCore.createCompilerError(
                  33,
                  key2.loc
                )
              );
              return true;
            }
          }
        });
      }
      if (context.prefixIdentifiers) {
        value && removeIdentifiers(value);
        key && removeIdentifiers(key);
        index && removeIdentifiers(index);
      }
      shared.extend(
        clearExpr(cloneSourceExpr),
        isScopedSlot ? createVSlotCallExpression(
          node.slotComponent,
          vForScope,
          context
        ) : createVForCallExpression(vForScope, context)
      );
      popScope();
    };
  }
);
function clearExpr(expr) {
  Object.keys(expr).forEach((key) => {
    delete expr[key];
  });
  return expr;
}
function parseAlias(babelExpr, exprCode, fallback) {
  if (types.isIdentifier(babelExpr)) {
    return exprCode;
  }
  return fallback;
}
function parseVForScope(currentScope) {
  while (currentScope) {
    if (isVForScope(currentScope) && !isScopedSlotVFor(currentScope)) {
      return currentScope;
    }
    currentScope = currentScope.parent;
  }
}
function findVIfReferencedScope(node, currentScope, context) {
  if (!currentScope) {
    return;
  }
  const vForScope = parseVForScope(currentScope);
  if (!vForScope) {
    return;
  }
  if (!node.children.find((item) => checkVIfReferenced(item, vForScope, context))) {
    return findVIfReferencedScope(node, currentScope.parent, context);
  }
  return vForScope;
}
function checkVIfReferenced(node, vForScope, context) {
  if (!qinjsCliShared.isElementNode(node)) {
    return false;
  }
  if (compilerCore.findDir(node, "for")) {
    return false;
  }
  const ifDir = compilerCore.findDir(node, "if");
  if (ifDir) {
    return checkDirReferenced(ifDir.exp, vForScope, context);
  }
  const elseIfDir = compilerCore.findDir(node, "else-if");
  if (elseIfDir) {
    return checkDirReferenced(elseIfDir.exp, vForScope, context);
  }
  return !!node.children.find(
    (item) => checkVIfReferenced(item, vForScope, context)
  );
}
function checkDirReferenced(node, vForScope, context) {
  if (node) {
    const babelNode = parseExpr(node, context);
    if (babelNode && isReferencedByIds(babelNode, vForScope.locals)) {
      return true;
    }
  }
  return false;
}
function findVForLocals({ value, key, index }) {
  const ids = [];
  if (value) {
    findIds(value, ids);
  }
  if (key) {
    findIds(key, ids);
  }
  if (index) {
    findIds(index, ids);
  }
  return ids;
}
function findIds(exp, ids) {
  if (shared.isString(exp)) {
    ids.push(exp);
  } else if (exp.identifiers) {
    exp.identifiers.forEach((id) => ids.push(id));
  } else if (exp.type === 4) {
    ids.push(exp.content);
  }
}
const forAliasRE = /([\s\S]*?)\s+(?:in|of)\s+([\s\S]*)/;
const forIteratorRE = /,([^,\}\]]*)(?:,([^,\}\]]*))?$/;
const stripParensRE = /^\(|\)$/g;
function parseForExpression(input, context) {
  const loc = input.loc;
  const exp = input.content;
  const inMatch = exp.match(forAliasRE);
  if (!inMatch)
    return;
  const [, LHS, RHS] = inMatch;
  const result = {
    source: createAliasExpression(
      loc,
      RHS.trim(),
      exp.indexOf(RHS, LHS.length)
    ),
    value: compilerCore.createSimpleExpression("v" + context.scopes.vFor),
    key: compilerCore.createSimpleExpression("k" + context.scopes.vFor),
    index: compilerCore.createSimpleExpression("i" + context.scopes.vFor),
    tagType: 0
  };
  if (context.prefixIdentifiers) {
    result.source = processExpression(
      result.source,
      context
    );
  }
  let valueContent = LHS.trim().replace(stripParensRE, "").trim();
  const trimmedOffset = LHS.indexOf(valueContent);
  const iteratorMatch = valueContent.match(forIteratorRE);
  if (iteratorMatch) {
    valueContent = valueContent.replace(forIteratorRE, "").trim();
    const keyContent = iteratorMatch[1].trim();
    let keyOffset;
    if (keyContent) {
      keyOffset = exp.indexOf(keyContent, trimmedOffset + valueContent.length);
      result.key = createAliasExpression(loc, keyContent, keyOffset);
      if (context.prefixIdentifiers) {
        result.key = processExpression(result.key, context, true);
      }
    }
    if (iteratorMatch[2]) {
      const indexContent = iteratorMatch[2].trim();
      if (indexContent) {
        result.index = createAliasExpression(
          loc,
          indexContent,
          exp.indexOf(
            indexContent,
            result.key ? keyOffset + keyContent.length : trimmedOffset + valueContent.length
          )
        );
        if (context.prefixIdentifiers) {
          result.index = processExpression(result.index, context, true);
        }
      }
    }
  }
  if (valueContent) {
    result.value = createAliasExpression(loc, valueContent, trimmedOffset);
    if (context.prefixIdentifiers) {
      result.value = processExpression(result.value, context, true);
    }
  }
  return result;
}
function createAliasExpression(range, content, offset) {
  return compilerCore.createSimpleExpression(
    content,
    false,
    qinjsCliShared.getInnerRange(range, offset, content.length)
  );
}
function createVForCallExpression(vForScope, context) {
  return types.callExpression(types.identifier(context.helperString(V_FOR)), [
    vForScope.sourceExpr,
    createVForArrowFunctionExpression(vForScope)
  ]);
}
function createVForArrowFunctionExpression({
  valueExpr,
  keyExpr,
  indexExpr,
  properties
}) {
  const params = [];
  if (valueExpr) {
    params.push(valueExpr);
  }
  if (keyExpr) {
    params.push(keyExpr);
  }
  if (indexExpr) {
    params.push(indexExpr);
  }
  return types.arrowFunctionExpression(
    params,
    types.blockStatement([types.returnStatement(types.objectExpression(properties))])
  );
}

const fnExpRE = /^\s*([\w$_]+|(async\s*)?\([^)]*?\))\s*=>|^\s*(async\s+)?function(?:\s+[\w$]+)?\s*\(/;
const transformOn = (dir, node, _context, augmentor) => {
  const context = _context;
  const { loc, modifiers, arg } = dir;
  if (!dir.exp && !modifiers.length) {
    context.onError(
      compilerCore.createCompilerError(35, loc)
    );
  }
  let eventName;
  if (arg.type === 4) {
    if (arg.isStatic) {
      const rawName = arg.content;
      eventName = compilerCore.createSimpleExpression(
        shared.toHandlerKey(shared.camelize(rawName)),
        true,
        arg.loc
      );
    } else {
      eventName = compilerCore.createCompoundExpression([
        // `${context.helperString(TO_HANDLER_KEY)}(`,
        arg
        // `)`,
      ]);
    }
  } else {
    eventName = arg;
    eventName.children.unshift(`${context.helperString(compilerCore.TO_HANDLER_KEY)}(`);
    eventName.children.push(`)`);
  }
  let exp = dir.exp;
  if (exp && !exp.content.trim()) {
    exp = void 0;
  }
  let shouldCache = context.cacheHandlers && !exp && !context.inVOnce;
  if (exp) {
    const isMemberExp = compilerCore.isMemberExpression(exp.content, context);
    const isInlineStatement = !(isMemberExp || fnExpRE.test(exp.content));
    const hasMultipleStatements = exp.content.includes(`;`);
    if (context.prefixIdentifiers) {
      isInlineStatement && context.addIdentifiers(`$event`);
      exp = dir.exp = processExpression(
        exp,
        context,
        false,
        hasMultipleStatements
      );
      isInlineStatement && context.removeIdentifiers(`$event`);
      shouldCache = context.cacheHandlers && // unnecessary to cache inside v-once
      !context.inVOnce && // runtime constants don't need to be cached
      // (this is analyzed by compileScript in SFC <script setup>)
      !(exp.type === 4 && exp.constType > 0) && // #1541 bail if this is a member exp handler passed to a component -
      // we need to use the original function to preserve arity,
      // e.g. <transition> relies on checking cb.length to determine
      // transition end handling. Inline function is ok since its arity
      // is preserved even when cached.
      !(isMemberExp && node.tagType === 1) && // bail if the function references closure variables (v-for, v-slot)
      // it must be passed fresh to avoid stale values.
      !compilerCore.hasScopeRef(exp, context.identifiers) && // wxs event
      !isFilterExpr(exp, context);
      if (shouldCache && isMemberExp) {
        if (exp.type === 4) {
          exp.content = `${exp.content} && ${exp.content}(...args)`;
        } else {
          exp.children = [
            ...exp.children,
            ` && `,
            ...exp.children,
            `(...args)`
          ];
        }
      }
    }
    if (isInlineStatement || shouldCache && isMemberExp) {
      exp = compilerCore.createCompoundExpression([
        `${isInlineStatement ? context.isTS ? `($event: any)` : `$event` : `${context.isTS ? `
//@ts-ignore
` : ``}(...args)`} => ${hasMultipleStatements ? `{` : `(`}`,
        exp,
        hasMultipleStatements ? `}` : `)`
      ]);
    }
  }
  let ret = {
    props: [
      compilerCore.createObjectProperty(
        eventName,
        exp || compilerCore.createSimpleExpression(`() => {}`, false, loc)
      )
    ]
  };
  if (augmentor) {
    ret = augmentor(ret);
  }
  if (shouldCache) {
    ret.props[0].value = wrapperVOn(
      ret.props[0].value,
      node,
      context
    );
  } else {
    ret.props[0].value = wrapperVOn(
      ret.props[0].value,
      node,
      context
    );
  }
  ret.props.forEach((p) => p.key.isHandlerKey = true);
  return ret;
};
function isFilterExpr(value, context) {
  if (context.filters.length && value.type === 8) {
    const firstChild = value.children[0];
    if (firstChild.type === 4 && context.filters.includes(firstChild.content)) {
      return true;
    }
  }
  return false;
}
function wrapperVOn(value, node, context) {
  if (isBuiltInIdentifier(value)) {
    return value;
  }
  if (isFilterExpr(value, context)) {
    return value;
  }
  const keys = [];
  if (context.miniProgram.event?.key && context.inVFor) {
    let keyProp = compilerCore.findProp(node, "key");
    if (!keyProp) {
      const vForScope = parseVForScope(context.currentScope);
      if (vForScope) {
        keyProp = compilerCore.findProp(vForScope.node, "key");
      }
    }
    if (keyProp && qinjsCliShared.isDirectiveNode(keyProp) && keyProp.exp) {
      const keyCode = genExpr(keyProp.exp);
      if (keyCode) {
        keys.push(",");
        keys.push(genExpr(keyProp.exp));
      }
    }
  }
  return compilerCore.createCompoundExpression([
    `${context.helperString(V_ON)}(`,
    value,
    ...keys,
    `)`
  ]);
}

const transformModel = (dir, node, _context) => {
  const context = _context;
  const baseResult = compilerCore.transformModel(dir, node, _context);
  if (!baseResult.props.length || node.tagType === 1) {
    return transformComponentVModel(
      baseResult.props,
      node,
      context
    );
  }
  if (dir.arg) {
    context.onError(
      compilerDom.createDOMCompilerError(
        56,
        dir.arg.loc
      )
    );
  }
  function checkDuplicatedValue() {
    const value = compilerCore.findProp(node, "value");
    if (value) {
      context.onError(
        compilerDom.createDOMCompilerError(
          58,
          value.loc
        )
      );
    }
  }
  const { tag } = node;
  if (tag === "input" || tag === "textarea") {
    checkDuplicatedValue();
  } else {
    context.onError(
      compilerDom.createDOMCompilerError(
        55,
        dir.loc
      )
    );
  }
  if (dir.modifiers.length) {
    const arg = dir.arg;
    const modifiers = dir.modifiers.map((m) => (compilerCore.isSimpleIdentifier(m) ? m : JSON.stringify(m)) + `: true`).join(`, `);
    const modifiersKey = arg ? compilerCore.isStaticExp(arg) ? `${arg.content}Modifiers` : compilerCore.createCompoundExpression([arg, ' + "Modifiers"']) : `modelModifiers`;
    baseResult.props.push(
      compilerCore.createObjectProperty(
        modifiersKey,
        compilerCore.createSimpleExpression(
          `{ ${modifiers} }`,
          false,
          dir.loc,
          2
        )
      )
    );
  }
  return transformElementVModel(
    baseResult.props,
    node,
    context
  );
};
function findInputDirectiveNode(props) {
  return props.find(
    (prop) => prop.type === 7 && prop.name === "on" && prop.arg?.type === 4 && prop.arg.content === "input"
  );
}
function transformElementVModel(props, node, context) {
  const dirs = transformVModel(props, node, context, {
    isComponent: false,
    binding: "value",
    event: "input",
    formatEventCode(code) {
      return code.replace(/=\s\$event/g, `= $event.detail.value`);
    }
  });
  if (dirs.length === 2) {
    const inputExp = findDatasetEventOpts(node);
    if (inputExp) {
      inputExp.children[2] = combineVOn(
        dirs[1].exp,
        inputExp.children[2],
        node,
        context
      );
      dirs.length = 1;
    } else {
      const inputDir = findInputDirectiveNode(node.props);
      if (inputDir && inputDir.exp) {
        inputDir.exp = combineVOn(dirs[1].exp, inputDir.exp, node, context);
        dirs.length = 1;
      }
    }
  }
  return { props: dirs };
}
function findDatasetEventOpts(node) {
  const eventOptsProp = compilerCore.findProp(
    node,
    qinjsCliShared.ATTR_DATASET_EVENT_OPTS,
    true,
    false
  );
  if (!eventOptsProp) {
    return;
  }
  const { exp } = eventOptsProp;
  if (exp?.type !== 8) {
    return;
  }
  for (let i = 0; i < exp.children.length; i++) {
    const childExp = exp.children[i];
    if (shared.isSymbol(childExp) || shared.isString(childExp)) {
      continue;
    }
    if (childExp.type !== 8) {
      continue;
    }
    if (childExp.children[0] !== `'input'`) {
      continue;
    }
    const inputExp = childExp.children[2];
    if (shared.isSymbol(inputExp) || shared.isString(inputExp) || inputExp.type !== 8) {
      continue;
    }
    return childExp;
  }
}
function parseVOn(exp, context) {
  return genExpr(exp).slice(context.helperString(V_ON).length + 1, -1);
}
function combineVOn(exp1, exp2, node, context) {
  return wrapperVOn(
    compilerCore.createCompoundExpression([
      `[`,
      parseVOn(exp1, context),
      ",",
      parseVOn(exp2, context),
      `]`
    ]),
    node,
    context
  );
}
function transformComponentVModel(props, node, context) {
  return {
    props: transformVModel(props, node, context, {
      isComponent: true,
      formatEventCode(code) {
        return code;
      }
    })
  };
}
function transformVModel(props, node, context, {
  isComponent,
  binding,
  event,
  formatEventCode
}) {
  if (props.length < 2) {
    return [];
  }
  const { key: modelValueArg, value: modelValeExpr } = props[0];
  const { key: onUpdateArg, value: onUpdateExpr } = props[1];
  if (modelValueArg.type !== 4) {
    return [];
  }
  if (onUpdateArg.type !== 4 || !onUpdateArg.content.startsWith("onUpdate:")) {
    return [];
  }
  const vBindModelValue = qinjsCliShared.createBindDirectiveNode(
    binding || modelValueArg.content,
    genExpr(modelValeExpr)
  );
  const modifiers = parseVModelModifiers(props[2]);
  const vOnValue = onUpdateExpr.type === 20 ? onUpdateExpr.value : onUpdateExpr;
  const vOnUpdate = qinjsCliShared.createOnDirectiveNode(
    event || shared.camelize(onUpdateArg.content.replace("onUpdate:", "update-")),
    formatEventCode(
      genExpr(
        wrapperVOn(
          modifiers ? wrapperVModelModifiers(vOnValue, modifiers, context, isComponent) : vOnValue,
          node,
          context
        )
      )
    )
  );
  return [vBindModelValue, vOnUpdate];
}
function parseVModelModifiers(property) {
  if (property && qinjsCliShared.isSimpleExpressionNode(property.key) && property.key.content.endsWith("Modifiers") && qinjsCliShared.isSimpleExpressionNode(property.value)) {
    return property.value.content;
  }
}
function wrapperVModelModifiers(exp, modifiers, context, isComponent = false) {
  return compilerCore.createCompoundExpression([
    `${context.helperString(WITH_MODEL_MODIFIERS)}(`,
    exp,
    ",",
    modifiers,
    `${isComponent ? `, true` : ``}`,
    `)`
  ]);
}

const transformElement = (node, context) => {
  return function postTransformElement() {
    node = context.currentNode;
    if (!(node.type === 1 && (node.tagType === 0 || node.tagType === 1))) {
      return;
    }
    if (node.tagType === 1) {
      processComponent(node, context);
    }
    if (context.scopeId) {
      addScopeId(node, context.scopeId);
    }
    const { props } = node;
    if (props.length > 0) {
      processProps(node, context);
    }
  };
};
function addScopeId(node, scopeId) {
  return qinjsCliShared.addStaticClass(node, scopeId);
}
function processComponent(node, context) {
  const { tag } = node;
  if (context.bindingComponents[tag]) {
    return;
  }
  if (qinjsShared.isComponentTag(tag)) {
    return context.onError(
      createMPCompilerError(
        MPErrorCodes.X_DYNAMIC_COMPONENT_NOT_SUPPORTED,
        node.loc
      )
    );
  }
  if (compilerCore.findDir(node, "is")) {
    return context.onError(
      createMPCompilerError(MPErrorCodes.X_V_IS_NOT_SUPPORTED, node.loc)
    );
  }
  const builtIn = compilerCore.isCoreComponent(tag) || context.isBuiltInComponent(tag);
  if (builtIn) {
    return context.onError(
      createMPCompilerError(MPErrorCodes.X_NOT_SUPPORTED, node.loc, tag)
    );
  }
  const fromSetup = resolveSetupReference(tag, context);
  if (fromSetup) {
    return context.bindingComponents[tag] = {
      name: fromSetup,
      type: BindingComponentTypes.SETUP
    };
  }
  const dotIndex = tag.indexOf(".");
  if (dotIndex > 0) {
    return context.onError(
      createMPCompilerError(MPErrorCodes.X_NOT_SUPPORTED, node.loc, tag)
    );
  }
  if (context.selfName && shared.capitalize(shared.camelize(tag)) === context.selfName) {
    return context.bindingComponents[tag] = {
      name: compilerCore.toValidAssetId(tag, `component`),
      type: BindingComponentTypes.SELF
    };
  }
  context.bindingComponents[tag] = {
    name: compilerCore.toValidAssetId(tag, `component`),
    type: BindingComponentTypes.UNKNOWN
  };
  context.helper(compilerCore.RESOLVE_COMPONENT);
}
function resolveSetupReference(name, context) {
  const bindings = context.bindingMetadata;
  if (!bindings || bindings.__isScriptSetup === false) {
    return;
  }
  const camelName = shared.camelize(name);
  const PascalName = shared.capitalize(camelName);
  const checkType = (type) => {
    if (bindings[name] === type) {
      return name;
    }
    if (bindings[camelName] === type) {
      return camelName;
    }
    if (bindings[PascalName] === type) {
      return PascalName;
    }
  };
  const fromConst = checkType(
    "setup-const"
    /* BindingTypes.SETUP_CONST */
  ) || checkType(
    "setup-reactive-const"
    /* BindingTypes.SETUP_REACTIVE_CONST */
  );
  if (fromConst) {
    return context.inline ? (
      // in inline mode, const setup bindings (e.g. imports) can be used as-is
      fromConst
    ) : `$setup[${JSON.stringify(fromConst)}]`;
  }
  const fromMaybeRef = checkType(
    "setup-let"
    /* BindingTypes.SETUP_LET */
  ) || checkType(
    "setup-ref"
    /* BindingTypes.SETUP_REF */
  ) || checkType(
    "setup-maybe-ref"
    /* BindingTypes.SETUP_MAYBE_REF */
  );
  if (fromMaybeRef) {
    return context.inline ? (
      // setup scope bindings that may be refs need to be unrefed
      `${context.helperString(compilerCore.UNREF)}(${fromMaybeRef})`
    ) : `$setup[${JSON.stringify(fromMaybeRef)}]`;
  }
}
function processProps(node, context, props = node.props) {
  const { tag } = node;
  const isComponent = node.tagType === 1;
  const isPluginComponent = isComponent && context.isMiniProgramComponent(node.tag) === "plugin";
  for (let i = 0; i < props.length; i++) {
    const prop = props[i];
    if (prop.type === 7) {
      const { name, arg, loc } = prop;
      const isVBind = name === "bind";
      const isVOn = name === "on";
      if (name === "slot") {
        if (!isComponent) {
          context.onError(
            compilerCore.createCompilerError(40, loc)
          );
        }
        continue;
      }
      if (name === "once" || name === "memo") {
        continue;
      }
      if (name === "is" || isVBind && compilerCore.isStaticArgOf(arg, "is") && qinjsShared.isComponentTag(tag)) {
        continue;
      }
      if (isVBind || isVOn) {
        if (!arg) {
          if (isVOn) {
            context.onError(
              createMPCompilerError(MPErrorCodes.X_V_ON_NO_ARGUMENT, loc)
            );
          }
          if (isVBind && (!isComponent || isPluginComponent)) {
            context.onError(
              createMPCompilerError(MPErrorCodes.X_V_BIND_NO_ARGUMENT, loc)
            );
          }
          continue;
        }
        if (!(arg.type === 4 && arg.isStatic)) {
          if (isVOn) {
            context.onError(
              createMPCompilerError(MPErrorCodes.X_V_ON_DYNAMIC_EVENT, loc)
            );
          }
          if (isVBind && (!isComponent || isPluginComponent)) {
            context.onError(
              createMPCompilerError(
                MPErrorCodes.X_V_BIND_DYNAMIC_ARGUMENT,
                loc
              )
            );
          }
          continue;
        }
      }
      const directiveTransform = context.directiveTransforms[name];
      if (name !== "model" && directiveTransform) {
        const { props: props2 } = directiveTransform(prop, node, context);
        if (props2.length) {
          prop.exp = props2[0].value;
        }
      }
    }
  }
  const transformVModel = context.directiveTransforms.model || transformModel;
  processVModel(node, transformVModel, context);
}
function processVModel(node, transformVModel, context) {
  const { props } = node;
  const dirs = [];
  for (let i = 0; i < props.length; i++) {
    const prop = props[i];
    if (prop.type === 7 && prop.name === "model") {
      dirs.push(
        ...transformVModel(prop, node, context).props
      );
      props.splice(i, 1);
      i--;
    }
  }
  if (dirs.length) {
    props.push(...dirs);
  }
}

function rewriteSlot(node, context) {
  let slotName = `"${qinjsShared.SLOT_DEFAULT_NAME}"`;
  let hasOtherDir = false;
  const nonNameProps = [];
  const { props } = node;
  if (!compilerCore.findProp(node, "name")) {
    props.unshift(qinjsCliShared.createAttributeNode("name", "default"));
  }
  for (let i = 0; i < props.length; i++) {
    const p = props[i];
    if (p.type === 6) {
      if (p.value) {
        if (p.name === "name") {
          p.value.content = qinjsShared.dynamicSlotName(p.value.content);
          slotName = JSON.stringify(p.value.content);
        } else {
          p.name = shared.camelize(p.name);
          nonNameProps.push(p);
        }
      }
    } else {
      if (p.name !== "bind") {
        hasOtherDir = true;
      }
      if (p.name === "bind" && compilerCore.isStaticArgOf(p.arg, "name")) {
        if (p.exp) {
          slotName = compilerCore.createCompoundExpression([
            context.helperString(DYNAMIC_SLOT) + "(",
            p.exp,
            ")"
          ]);
          let slotKey;
          const keys = parseVForKeyAlias(context);
          if (keys.length) {
            const babelNode = parseExpr(p.exp, context);
            if (babelNode && isReferencedByIds(babelNode, keys)) {
              slotKey = parseScopedSlotKey(context);
            }
          }
          p.exp = rewriteExpression(
            compilerCore.createCompoundExpression([
              context.helperString(DYNAMIC_SLOT) + "(",
              p.exp,
              slotKey ? `+'-'+` + slotKey : "",
              ")"
            ]),
            context
          );
        }
      } else {
        if (p.name === "bind" && p.arg && compilerCore.isStaticExp(p.arg)) {
          p.arg.content = shared.camelize(p.arg.content);
        }
        nonNameProps.push(p);
      }
    }
  }
  if (hasOtherDir) {
    context.onError(
      compilerCore.createCompilerError(
        36,
        node.loc
      )
    );
  }
  if (nonNameProps.length > 0) {
    processProps(node, context, nonNameProps);
    const properties = [];
    nonNameProps.forEach((prop) => {
      const property = transformProperty(prop);
      property && properties.push(property);
    });
    if (properties.length) {
      transformScopedSlotName(node, context);
      const vForIndexAlias = parseVForIndexAlias(context);
      rewriteExpression(
        compilerCore.createCompoundExpression([
          context.helperString(RENDER_SLOT) + "(",
          slotName,
          ",",
          `{${properties.join(",")}}`,
          `${vForIndexAlias ? "," + parseScopedSlotKey(context) : ""}`,
          ")"
        ]),
        context
      );
    } else {
      if (slotName === `"${qinjsShared.SLOT_DEFAULT_NAME}"`) {
        removeAttribute(node, "name");
      }
    }
  } else {
    if (slotName === `"${qinjsShared.SLOT_DEFAULT_NAME}"`) {
      removeAttribute(node, "name");
    }
  }
}
function parseVForIndexAlias(context) {
  let { currentScope } = context;
  while (currentScope) {
    if (isVForScope(currentScope) && !isScopedSlotVFor(currentScope)) {
      return currentScope.indexAlias;
    }
    currentScope = currentScope.parent;
  }
}
function transformScopedSlotName(node, context) {
  if (!context.miniProgram.slot.dynamicSlotNames) {
    return;
  }
  const slotKey = parseScopedSlotKey(context);
  if (!slotKey) {
    return;
  }
  const nameProps = compilerCore.findProp(node, "name");
  if (nameProps && nameProps.type === 6 && nameProps.value) {
    const { props } = node;
    props.splice(
      props.indexOf(nameProps),
      1,
      createScopedSlotDirectiveNode(nameProps.value.content, slotKey, context)
    );
  }
}
function createScopedSlotDirectiveNode(name, slotKey, context) {
  const dir = qinjsCliShared.createBindDirectiveNode(
    "name",
    rewriteExpression(compilerCore.createSimpleExpression(`"${name}-"+` + slotKey), context).content
  );
  dir.slotName = name;
  return dir;
}
function parseVForKeyAlias(context) {
  let { currentScope } = context;
  const keys = [];
  while (currentScope) {
    if (isVForScope(currentScope) && !isScopedSlotVFor(currentScope)) {
      keys.push(currentScope.keyAlias);
    }
    currentScope = currentScope.parent;
  }
  return keys;
}
function parseVForIndexes(context) {
  let { currentScope } = context;
  const indexes = [];
  while (currentScope) {
    if (isVForScope(currentScope) && !isScopedSlotVFor(currentScope)) {
      indexes.push(currentScope.indexAlias);
    }
    currentScope = currentScope.parent;
  }
  return indexes;
}
function parseSlotKeyByVForIndexes(indexes) {
  return indexes.reverse().join(`+'-'+`);
}
function parseScopedSlotKey(context) {
  const indexes = parseVForIndexes(context);
  const inFor = !!indexes.length;
  if (inFor) {
    return parseSlotKeyByVForIndexes(indexes);
  }
}
function transformProperty(dir, _) {
  if (qinjsCliShared.isDirectiveNode(dir)) {
    if (!dir.arg || !dir.exp) {
      return;
    }
    const isStaticArg = dir.arg.type === 4 && dir.arg.isStatic;
    if (isStaticArg) {
      return `${dir.arg.content}:${genExpr(dir.exp)}`;
    }
    return `[${genExpr(dir.arg)}||'']:${genExpr(dir.exp)}`;
  }
  if (dir.value) {
    return `${dir.name}:${genExpr(dir.value)}`;
  }
}

function rewriteRef(node, context) {
  const vueIdProp = compilerCore.findProp(node, ATTR_VUE_ID);
  if (!vueIdProp) {
    return;
  }
  const refProp = compilerCore.findProp(node, "u-" + qinjsCliShared.VUE_REF) || compilerCore.findProp(node, "u-" + qinjsCliShared.VUE_REF_IN_FOR);
  if (!refProp) {
    return;
  }
  if (compilerCore.findProp(node, "ref")) {
    const code = parseAlipayRefCode(refProp, context);
    if (code && context.inline && !qinjsCliShared.isDirectiveNode(refProp)) {
      refProp.value.content = code;
      const refPropIndex = node.props.findIndex((prop) => prop === refProp);
      node.props.splice(
        refPropIndex,
        1,
        qinjsCliShared.createBindDirectiveNode(refProp.name, code)
      );
    }
  } else {
    rewriteRefProp(refProp, vueIdProp, context);
  }
}
function parseRef(prop, context) {
  let expr;
  let refKey = "";
  const isDir = qinjsCliShared.isDirectiveNode(prop);
  if (isDir) {
    if (prop.exp) {
      expr = parseExpr(prop.exp, context, prop.exp);
    }
  } else {
    const { value } = prop;
    if (value && value.content) {
      if (context.inline && context.bindingMetadata[value.content]) {
        expr = types.identifier(value.content);
        refKey = value.content;
      } else {
        expr = types.stringLiteral(value.content);
      }
    }
  }
  return { expr, refKey };
}
function parseRefCode(prop, context) {
  const { expr, refKey } = parseRef(prop, context);
  if (!expr) {
    return { code: "", refKey };
  }
  return { code: genBabelExpr(expr), refKey };
}
function rewriteRefProp(prop, vueIdProp, context) {
  let id = "";
  if (qinjsCliShared.isDirectiveNode(vueIdProp)) {
    const vueIdExpr = parseExpr(vueIdProp.exp, context, vueIdProp.exp);
    if (vueIdExpr) {
      id = genBabelExpr(vueIdExpr);
    }
  } else {
    id = `'${vueIdProp.value.content}'`;
  }
  if (!id) {
    return;
  }
  const { code, refKey } = parseRefCode(prop, context);
  const opts = /* @__PURE__ */ Object.create(null);
  if (refKey) {
    opts.k = refKey;
  }
  if (context.inVFor) {
    opts.f = 1;
  }
  parseExprWithRewrite(
    context.helperString(SET_REF) + "(" + code + ", " + id + (Object.keys(opts).length ? ", " + JSON.stringify(opts) : "") + ")",
    prop.loc,
    context
  );
}
function parseAlipayRefCode(prop, context) {
  let expr;
  const isDir = qinjsCliShared.isDirectiveNode(prop);
  if (isDir) {
    if (prop.exp) {
      expr = parseExpr(prop.exp, context, prop.exp);
    }
  } else {
    if (prop.value?.content) {
      expr = context.inline ? processInlineRef(prop, context) : types.stringLiteral(prop.value.content);
    }
  }
  if (!expr) {
    return;
  }
  return genBabelExpr(expr);
}
function processInlineRef(prop, context) {
  const properties = [];
  const { refKey } = parseRef(prop, context);
  properties.push(
    types.objectProperty(types.identifier("r"), types.identifier(prop.value.content))
  );
  if (refKey) {
    properties.push(types.objectProperty(types.identifier("k"), types.stringLiteral(refKey)));
  }
  if (context.inVFor) {
    properties.push(types.objectProperty(types.identifier("f"), types.numericLiteral(1)));
  }
  return types.arrowFunctionExpression([], types.objectExpression(properties));
}

function isIfElementNode(node) {
  return !!node.vIf;
}
const transformIf = createStructuralDirectiveTransform(
  /^(if|else|else-if)$/,
  (node, dir, context) => {
    return processIf(node, dir, context, (ifNode, branch, isRoot) => {
      const { currentScope: parentScope, popScope } = context;
      const ifOptions = {
        name: dir.name
      };
      branch.vIf = ifOptions;
      const condition = dir.exp ? parseExpr(dir.exp, context) : void 0;
      const vIfScope = context.addVIfScope({
        name: dir.name,
        condition
      });
      if (condition) {
        if (!isStaticLiteral(condition)) {
          ifOptions.condition = rewriteExpression(
            dir.exp,
            context,
            condition,
            parentScope
          ).content;
        } else {
          ifOptions.condition = dir.exp.content;
        }
      }
      return () => {
        if (isRoot) {
          parentScope.properties.push(createVIfSpreadElement(vIfScope));
        } else {
          const vIfSpreadElement = findVIfSpreadElement(parentScope);
          if (!vIfSpreadElement) {
            popScope();
            return context.onError(
              compilerCore.createCompilerError(
                30,
                node.loc
              )
            );
          }
          let alternate = createObjectExpression([]);
          if (dir.name === "else-if") {
            alternate = createVIfConditionalExpression(vIfScope);
          } else if (dir.name === "else") {
            alternate = createObjectExpression(vIfScope.properties);
          }
          findVIfConditionalExpression(
            vIfSpreadElement.argument
          ).alternate = alternate;
        }
        popScope();
      };
    });
  }
);
function processIf(node, dir, context, processCodegen) {
  if (dir.name !== "else" && (!dir.exp || !dir.exp.content.trim())) {
    const loc = dir.exp ? dir.exp.loc : node.loc;
    context.onError(
      compilerCore.createCompilerError(28, dir.loc)
    );
    dir.exp = compilerCore.createSimpleExpression(`true`, false, loc);
  }
  if (context.prefixIdentifiers && dir.exp) {
    dir.exp = processExpression(dir.exp, context);
  }
  if (dir.name === "if") {
    const ifNode = {
      type: 9,
      loc: node.loc,
      branches: [node]
    };
    context.replaceNode(ifNode);
    if (processCodegen) {
      return processCodegen(ifNode, node, true);
    }
  } else {
    const siblings = context.parent.children;
    let i = siblings.indexOf(node);
    while (i-- >= -1) {
      const sibling = siblings[i];
      if (sibling && sibling.type === 3) {
        context.removeNode(sibling);
        continue;
      }
      if (sibling && sibling.type === 2 && !sibling.content.trim().length) {
        context.removeNode(sibling);
        continue;
      }
      if (sibling && sibling.type === 9) {
        if (dir.name === "else-if" && sibling.branches[sibling.branches.length - 1].vIf.condition === void 0) {
          context.onError(
            compilerCore.createCompilerError(
              30,
              node.loc
            )
          );
        }
        context.removeNode();
        sibling.branches.push(node);
        const onExit = processCodegen && processCodegen(sibling, node, false);
        traverseNode(node, context);
        if (onExit)
          onExit();
        context.currentNode = null;
      } else {
        context.onError(
          compilerCore.createCompilerError(
            30,
            node.loc
          )
        );
      }
      break;
    }
  }
}
function findVIfSpreadElement({ properties }) {
  const len = properties.length;
  for (let i = len - 1; i >= 0; i--) {
    const prop = properties[i];
    if (types.isSpreadElement(prop)) {
      return prop;
    }
  }
}
function findVIfConditionalExpression(vIfConditionalExpression) {
  if (types.isConditionalExpression(vIfConditionalExpression.alternate)) {
    return findVIfConditionalExpression(vIfConditionalExpression.alternate);
  }
  return vIfConditionalExpression;
}

const transformComponent = (node, context) => {
  if (!qinjsCliShared.isUserComponent(node, context)) {
    return;
  }
  if (node.tag === "template") {
    return;
  }
  addComponentType(node, context);
  addVueId(node, context);
  processBooleanAttr(node);
  return function postTransformComponent() {
    context.vueIds.pop();
  };
};
function addComponentType(node, context) {
  if (!context.isMiniProgramComponent(node.tag)) {
    return;
  }
  node.props.push(qinjsCliShared.createAttributeNode(ATTR_COM_TYPE, "m"));
}
function addVueId(node, context) {
  let { hashId, scopes, currentScope, currentVueId } = context;
  if (!hashId) {
    return;
  }
  let vueId = hashId + "-" + scopes.vueId++;
  const indexs = [];
  while (currentScope) {
    if (isVForScope(currentScope)) {
      indexs.push(`+'-'+${currentScope.indexAlias}`);
    }
    currentScope = currentScope.parent;
  }
  const inFor = !!indexs.length;
  if (inFor) {
    vueId = `'${vueId}'` + indexs.reverse().join("");
  }
  context.vueIds.push(vueId);
  let value = vueId;
  if (currentVueId) {
    const isParentDynamic = currentVueId.includes("+");
    const isCurrentDynamic = vueId.includes("+");
    if (isParentDynamic || isCurrentDynamic) {
      value = `(${vueId})+','+(${isParentDynamic ? currentVueId : `'${currentVueId}'`})`;
    } else {
      value = vueId + "," + currentVueId;
    }
  }
  if (value.includes("+")) {
    return node.props.push(qinjsCliShared.createBindDirectiveNode(ATTR_VUE_ID, value));
  }
  return node.props.push(qinjsCliShared.createAttributeNode(ATTR_VUE_ID, value));
}
function processBooleanAttr({ props }) {
  props.forEach((prop, index) => {
    if (prop.type === 6 && typeof prop.value === "undefined") {
      props.splice(index, 1, qinjsCliShared.createBindDirectiveNode(prop.name, "true"));
    }
  });
}
const builtInProps$1 = [
  // 'id',
  "class",
  "style",
  ATTR_VUE_ID,
  ATTR_VUE_PROPS,
  ATTR_VUE_SLOTS,
  ATTR_VUE_REF,
  ATTR_VUE_REF_IN_FOR,
  ATTR_COM_TYPE,
  "eO",
  "e-o",
  "onVI",
  "ref",
  "slot",
  "key",
  "is"
];
function isComponentProp(name) {
  if (builtInProps$1.includes(name)) {
    return false;
  }
  if (name.startsWith("data-")) {
    return false;
  }
  return true;
}
function rewriteBinding({ tag, props }, context) {
  const isMiniProgramComponent = context.isMiniProgramComponent(tag);
  if (isMiniProgramComponent === "plugin" || isMiniProgramComponent === "dynamicLib") {
    return;
  }
  const createObjectProperty = isMiniProgramComponent ? (name, value) => types.objectProperty(types.identifier(shared.camelize(name)), value) : (name, value) => {
    const computed = !compilerCore.isSimpleIdentifier(name);
    return types.objectProperty(
      computed ? types.stringLiteral(name) : types.identifier(name),
      value,
      computed
    );
  };
  const properties = [];
  for (let i = 0; i < props.length; i++) {
    const prop = props[i];
    let isIdProp = false;
    if (qinjsCliShared.isAttributeNode(prop)) {
      const { name } = prop;
      isIdProp = name === "id";
      if (!isComponentProp(name)) {
        continue;
      }
      properties.push(
        createObjectProperty(name, types.stringLiteral(prop.value?.content || ""))
      );
    } else if (qinjsCliShared.isDirectiveNode(prop)) {
      if (prop.name !== "bind") {
        continue;
      }
      const { arg, exp } = prop;
      if (!exp) {
        continue;
      }
      if (!arg) {
        const spreadElement2 = createVBindSpreadElement(prop, context);
        if (spreadElement2) {
          properties.push(spreadElement2);
        }
      } else if (compilerCore.isStaticExp(arg)) {
        isIdProp = arg.content === "id";
        if (!isComponentProp(arg.content)) {
          continue;
        }
        const valueExpr = parseExpr(genExpr(exp), context, exp);
        if (!valueExpr) {
          continue;
        }
        properties.push(createObjectProperty(arg.content, valueExpr));
      } else {
        const leftExpr = parseExpr(genExpr(arg), context, exp);
        if (!leftExpr) {
          continue;
        }
        const valueExpr = parseExpr(genExpr(exp), context, exp);
        if (!valueExpr) {
          continue;
        }
        properties.push(
          types.objectProperty(
            types.logicalExpression("||", leftExpr, types.stringLiteral("")),
            valueExpr,
            true
          )
        );
      }
    }
    if (!isIdProp) {
      props.splice(i, 1);
      i--;
    }
  }
  if (properties.length) {
    props.push(
      qinjsCliShared.createBindDirectiveNode(
        ATTR_VUE_PROPS,
        genBabelExpr(types.objectExpression(properties))
      )
    );
  }
}
function createVBindSpreadElement(prop, context) {
  const { arg, exp } = prop;
  if (!exp) {
    return;
  }
  if (!arg) {
    const argument = parseExpr(genExpr(exp), context, exp);
    if (!argument) {
      return;
    }
    return types.spreadElement(argument);
  }
}
function isPropsBinding({ arg }) {
  return arg && arg.type === 4 && arg.content === ATTR_VUE_PROPS;
}
function rewritePropsBinding(dir, node, context) {
  dir.exp = compilerCore.createSimpleExpression(
    genBabelExpr(
      rewirteWithHelper(
        RENDER_PROPS,
        parseExpr(dir.exp, context),
        dir.loc,
        context
      )
    ) + (isIfElementNode(node) && node.vIf.name === "else" ? `||''` : "")
  );
}

const transformIdentifier = (node, context) => {
  return function transformIdentifier2() {
    if (node.type === 5) {
      const content = node.content;
      let isFilter = false;
      if (content.type === 8) {
        const firstChild = content.children[0];
        isFilter = !shared.isString(firstChild) && !shared.isSymbol(firstChild) && firstChild.type === 4 && context.filters.includes(firstChild.content);
      }
      if (!isFilter) {
        node.content = rewriteExpression(
          compilerCore.createCompoundExpression([
            `${context.helperString(TO_DISPLAY_STRING)}(`,
            content,
            `)`
          ]),
          context
        );
      }
    } else if (compilerCore.isSlotOutlet(node)) {
      rewriteSlot(node, context);
    } else if (node.type === 1) {
      let hasClassBinding = false;
      let hasStyleBinding = false;
      rewriteRef(node, context);
      if (qinjsCliShared.isUserComponent(node, context)) {
        rewriteBinding(node, context);
      }
      const { props } = node;
      const virtualHost = !!(context.miniProgram.component?.mergeVirtualHostAttributes && context.rootNode === node);
      for (let i = 0; i < props.length; i++) {
        const dir = props[i];
        if (dir.type === 7) {
          const arg = dir.arg;
          if (arg) {
            if (!(arg.type === 4 && arg.isStatic)) {
              if (dir.name === "slot") {
                rewriteVSlot(dir, context);
              } else {
                props.splice(i, 1);
                i--;
                continue;
              }
            }
          }
          const exp = dir.exp;
          if (exp) {
            if (isBuiltIn(dir)) ; else if (isClassBinding(dir)) {
              hasClassBinding = true;
              rewriteClass(i, dir, props, virtualHost, context);
            } else if (isStyleBinding(dir)) {
              hasStyleBinding = true;
              rewriteStyle(i, dir, props, virtualHost, context);
            } else if (isPropsBinding(dir)) {
              rewritePropsBinding(dir, node, context);
            } else {
              dir.exp = rewriteExpression(exp, context);
            }
          }
        }
      }
      if (virtualHost) {
        if (!hasClassBinding) {
          hasClassBinding = true;
          props.push(createVirtualHostClass(props, context));
        }
        if (!hasStyleBinding) {
          hasStyleBinding = true;
          props.push(createVirtualHostStyle(props, context));
        }
      }
      if (hasClassBinding) {
        const staticClassIndex = findStaticClassIndex(props);
        if (staticClassIndex > -1) {
          props.splice(staticClassIndex, 1);
        }
      }
      if (hasStyleBinding) {
        const staticStyleIndex = findStaticStyleIndex(props);
        if (staticStyleIndex > -1) {
          props.splice(staticStyleIndex, 1);
        }
      }
    }
  };
};
const builtInProps = [ATTR_VUE_SLOTS];
function isBuiltIn({ arg, exp }) {
  return arg?.type === 4 && builtInProps.includes(arg.content) && exp?.type === 4;
}

function generate({ children }, {
  slot,
  event,
  scopeId,
  emitFile,
  filename,
  directive,
  lazyElement,
  isBuiltInComponent,
  isMiniProgramComponent,
  component
}) {
  const context = {
    slot,
    event,
    code: "",
    scopeId,
    directive,
    lazyElement,
    component,
    isBuiltInComponent,
    isMiniProgramComponent,
    push(code) {
      context.code += code;
    }
  };
  children.forEach((node) => {
    genNode(node, context);
  });
  emitFile({ type: "asset", fileName: filename, source: context.code });
}
function genNode(node, context) {
  switch (node.type) {
    case 9:
      return node.branches.forEach((node2) => {
        genNode(node2, context);
      });
    case 2:
      return genText(node, context);
    case 5:
      return genExpression(node.content, context);
    case 1:
      if (node.tagType === 2) {
        return genSlot(node, context);
      } else if (node.tagType === 1) {
        return genComponent(node, context);
      } else if (node.tagType === 3) {
        return genTemplate(node, context);
      } else if (isLazyElement(node, context)) {
        return genLazyElement(node, context);
      }
      return genElement(node, context);
  }
}
function genText(node, { push }) {
  push(node.content);
}
function genExpression(node, { push }) {
  push(`{{${genExpr(node)}}}`);
}
function genVIf(exp, { push, directive }) {
  push(` ${directive}if="{{${exp}}}"`);
}
function genVElseIf(exp, { push, directive }) {
  push(` ${directive}elif="{{${exp}}}"`);
}
function genVElse({ push, directive }) {
  push(` ${directive}else`);
}
function genVFor(node, { push, directive }) {
  const { sourceCode, valueAlias, indexAlias } = node.vFor;
  push(` ${directive}for="${sourceCode}"`);
  if (valueAlias) {
    push(` ${directive}for-item="${valueAlias}"`);
  }
  if (valueAlias === "index") {
    push(` ${directive}for-index="${indexAlias}"`);
  }
  const keyProp = compilerCore.findProp(node, "key", true);
  if (keyProp) {
    const key = keyProp.exp.content;
    push(` ${directive}key="${key.includes(".") ? key.split(".")[1] : key}"`);
    node.props.splice(node.props.indexOf(keyProp), 1);
  }
}
function genSlot(node, context) {
  node.props = node.props.filter((prop) => {
    if (prop.type === 6) {
      return prop.name === "name";
    } else if (prop.arg?.type === 4) {
      return prop.arg.content === "name";
    }
  });
  if (!node.children.length || context.slot.fallbackContent) {
    return genElement(node, context);
  }
  const { push } = context;
  const isVIfSlot = isIfElementNode(node);
  if (isVIfSlot) {
    push(`<block`);
    genVIfCode(node, context);
    push(`>`);
    delete node.vIf;
  }
  const children = node.children.slice();
  node.children.length = 0;
  push(`<block`);
  const nameProp = compilerCore.findProp(node, "name");
  let name = qinjsShared.SLOT_DEFAULT_NAME;
  if (nameProp) {
    if (qinjsCliShared.isAttributeNode(nameProp)) {
      if (nameProp.value?.content) {
        name = nameProp.value.content;
      }
    } else {
      if (nameProp.slotName) {
        name = nameProp.slotName;
      }
    }
  }
  genVIf(`$slots.${name}`, context);
  push(`>`);
  genElement(node, context);
  push(`</block>`);
  push(`<block`);
  genVElse(context);
  push(`>`);
  children.forEach((node2) => {
    genNode(node2, context);
  });
  push(`</block>`);
  if (isVIfSlot) {
    push(`</block>`);
  }
}
function genTemplate(node, context) {
  const slotProp = node.props.find(
    (prop) => prop.type === 7 && (prop.name === "slot" || prop.name === "bind" && prop.arg?.type === 4 && prop.arg.content === "slot")
  );
  if (slotProp && (slotProp.name === "bind" || findSlotName(slotProp))) {
    node.tag = "view";
  } else {
    node.tag = "block";
  }
  node.tagType = 0;
  if (slotProp && node.tag === "view" && !isForElementNode(node) && node.children.length === 1) {
    const child = node.children[0];
    if (qinjsCliShared.isElementNode(child) && !isForElementNode(child) && !compilerCore.isSlotOutlet(child)) {
      child.props.push(slotProp);
      return genElement(child, context);
    }
  }
  return genElement(node, context);
}
function genComponent(node, context) {
  if (context.component?.getPropertySync) {
    return genElement(node, context);
  }
  if (isIfElementNode(node) || isForElementNode(node)) {
    return genElement(node, context);
  }
  if (context.isMiniProgramComponent(node.tag)) {
    node.vIf = {
      name: "if",
      condition: "r0"
    };
    return genElement(node, context);
  }
  const prop = compilerCore.findProp(node, ATTR_VUE_PROPS);
  if (!prop) {
    return genElement(node, context);
  }
  node.vIf = {
    name: "if",
    condition: prop.exp.content
  };
  return genElement(node, context);
}
function isLazyElement(node, context) {
  if (!context.lazyElement) {
    return false;
  }
  let lazyProps;
  if (shared.isFunction(context.lazyElement)) {
    const res = context.lazyElement(node, context);
    if (!shared.isPlainObject(res)) {
      return res;
    }
    lazyProps = res[node.tag];
  } else {
    lazyProps = context.lazyElement[node.tag];
  }
  if (lazyProps === true) {
    return true;
  }
  if (!lazyProps) {
    return;
  }
  return node.props.some(
    (prop) => prop.type === 7 && lazyProps.find(
      (lazyProp) => {
        return prop.name === lazyProp.name && prop.arg?.type === 4 && lazyProp.arg.includes(prop.arg.content);
      }
    )
  );
}
function genLazyElement(node, context) {
  const { push } = context;
  if (!isIfElementNode(node)) {
    push(`<block`);
    genVIf(`r0`, context);
    push(`>`);
    genElement(node, context);
    push(`</block>`);
    return;
  }
  if (node.vIf.name !== "else") {
    return genElement(node, context);
  }
  push(`<block`);
  genVElse(context);
  push(`>`);
  node.vIf.name = "if";
  node.vIf.condition = "r0";
  genElement(node, context);
  push(`</block>`);
}
function genVIfCode(node, context) {
  const { name, condition } = node.vIf;
  if (name === "if") {
    genVIf(condition, context);
  } else if (name === "else-if") {
    genVElseIf(condition, context);
  } else if (name === "else") {
    genVElse(context);
  }
}
function genElement(node, context) {
  const { children, isSelfClosing, props } = node;
  let tag = node.tag;
  if (tag === "template") {
    if (compilerCore.findProp(node, "slot")) {
      tag = "view";
    } else {
      tag = "block";
    }
  }
  if (tag === "block" && props.length === 0 && !isIfElementNode(node) && !isForElementNode(node)) {
    return children.forEach((node2) => {
      genNode(node2, context);
    });
  }
  let virtualHost = false;
  if (qinjsCliShared.isUserComponent(node, context)) {
    tag = shared.hyphenate(tag);
    if (context.component?.normalizeName) {
      tag = context.component?.normalizeName(tag);
    }
    if (context.component?.mergeVirtualHostAttributes) {
      virtualHost = true;
    }
  }
  const { push } = context;
  const hasVIf = isIfElementNode(node);
  const hasVFor = isForElementNode(node);
  const hasVIfAndVFor = hasVIf && hasVFor;
  if (hasVIfAndVFor) {
    push(`<block`);
    genVIfCode(node, context);
    push(`>`);
  }
  push(`<${tag}`);
  if (!hasVIfAndVFor && hasVIf) {
    genVIfCode(node, context);
  }
  if (hasVFor) {
    genVFor(node, context);
  }
  if (props.length) {
    genElementProps(node, virtualHost, context);
  }
  if (isSelfClosing) {
    push(`/>`);
  } else {
    push(`>`);
    children.forEach((node2) => {
      genNode(node2, context);
    });
    push(`</${tag}>`);
  }
  if (hasVIfAndVFor) {
    push(`</block>`);
  }
}
function checkVirtualHostProps(name, virtualHost) {
  const names = [name];
  if (virtualHost) {
    const obj = {
      style: VIRTUAL_HOST_STYLE,
      class: VIRTUAL_HOST_CLASS
    };
    if (name in obj) {
      names.push(obj[name]);
    }
    return names;
  }
  return names;
}
function genElementProps(node, virtualHost, context) {
  node.props.forEach((prop) => {
    if (prop.type === 6) {
      const { value } = prop;
      if (value) {
        checkVirtualHostProps(prop.name, virtualHost).forEach((name) => {
          context.push(` ${name}="${value.content}"`);
        });
      } else {
        context.push(` ${prop.name}`);
      }
    } else {
      const { name } = prop;
      if (name === "on") {
        genOn(prop, node, context);
      } else {
        genDirectiveNode(prop, node, virtualHost, context);
      }
    }
  });
}
function genOn(prop, node, { push, event, isBuiltInComponent }) {
  const arg = prop.arg.content;
  const exp = prop.exp;
  const modifiers = prop.modifiers;
  const name = (event?.format || qinjsCliShared.formatMiniProgramEvent)(arg, {
    isCatch: modifiers.includes("stop") || modifiers.includes("prevent"),
    isCapture: modifiers.includes("capture"),
    isComponent: qinjsCliShared.isUserComponent(node, { isBuiltInComponent })
  });
  if (exp.isStatic) {
    push(` ${name}="${exp.content}"`);
  } else {
    push(` ${name}="{{${exp.content}}}"`);
  }
}
function genDirectiveNode(prop, node, virtualHost, context) {
  const { push, component } = context;
  if (prop.name === "slot") {
    if (prop.arg) {
      const arg = prop.arg;
      if (arg.isStatic) {
        const slotName = qinjsShared.dynamicSlotName(arg.content);
        if (slotName !== qinjsShared.SLOT_DEFAULT_NAME) {
          push(` slot="${slotName}"`);
        }
      } else {
        push(` slot="{{${arg.content}}}"`);
      }
    }
  } else if (prop.name === "show") {
    let hiddenPropName = "hidden";
    if (qinjsCliShared.isUserComponent(node, context) && component && component.vShow) {
      hiddenPropName = component.vShow;
    }
    push(
      ` ${hiddenPropName}="{{!${prop.exp.content}}}"`
    );
  } else if (prop.arg && prop.exp) {
    const arg = prop.arg.content;
    const exp = prop.exp.content;
    checkVirtualHostProps(arg, virtualHost).forEach((arg2) => {
      push(` ${arg2}="{{${exp}}}"`);
    });
  } else {
    if (prop.name !== "bind") {
      throw new Error(`unknown directive ` + JSON.stringify(prop));
    }
  }
}

const transformBind = (dir, _node, context) => {
  const { exp, modifiers, loc } = dir;
  const arg = dir.arg;
  if (arg.type !== 4) {
    arg.children.unshift(`(`);
    arg.children.push(`) || ""`);
  } else if (!arg.isStatic) {
    arg.content = `${arg.content} || ""`;
  }
  if (modifiers.includes("camel")) {
    if (arg.type === 4) {
      if (arg.isStatic) {
        arg.content = shared.camelize(arg.content);
      }
    }
  }
  if (modifiers.includes("prop")) {
    context.onWarn(
      createMPCompilerError(MPErrorCodes.X_V_BIND_MODIFIER_PROP, loc)
    );
  }
  if (modifiers.includes("attr")) {
    context.onWarn(
      createMPCompilerError(MPErrorCodes.X_V_BIND_MODIFIER_ATTR, loc)
    );
  }
  if (!exp || exp.type === 4 && !exp.content.trim()) {
    context.onError(
      compilerCore.createCompilerError(34, loc)
    );
    return {
      props: [compilerCore.createObjectProperty(arg, compilerCore.createSimpleExpression("", true, loc))]
    };
  }
  return {
    props: [compilerCore.createObjectProperty(arg, exp)]
  };
};

const transformRoot = (node, context) => {
  if (node.type !== 0) {
    return;
  }
  if (context.bindingCssVars.length) {
    node.children.forEach((child) => {
      if (child.type !== 1) {
        return;
      }
      addCssVars(child, context);
    });
  }
};
const CSS_VARS = "__cssVars()";
function addCssVars(node, context) {
  const styleProp = compilerCore.findProp(node, "style", true);
  if (!styleProp) {
    node.props.push(qinjsCliShared.createBindDirectiveNode("style", CSS_VARS));
  } else {
    if (styleProp.exp?.type === 4) {
      let expr = parseExpr(styleProp.exp.content, context);
      if (types.isArrayExpression(expr)) {
        expr.elements.push(types.identifier(CSS_VARS));
      } else {
        expr = types.arrayExpression([expr, types.identifier(CSS_VARS)]);
      }
      styleProp.exp.content = genBabelExpr(expr);
    }
  }
}

const transformTag = (node, _) => {
  if (!qinjsCliShared.isElementNode(node)) {
    return;
  }
  const newTag = qinjsCliShared.HTML_TO_MINI_PROGRAM_TAGS[node.tag];
  if (newTag) {
    node.tag = newTag;
    node.tagType = 0;
  }
};

const transformHtml = (node, _) => {
  if (!qinjsCliShared.isElementNode(node)) {
    return;
  }
  const dir = compilerCore.findDir(node, "html");
  if (!dir) {
    return;
  }
  node.props.splice(node.props.indexOf(dir), 1);
  if (node.tagType !== 0) {
    return;
  }
  node.isSelfClosing = false;
  node.children = [createRichText(node, dir)];
};
function createRichText(node, dir) {
  return {
    tag: "rich-text",
    type: 1,
    tagType: 0,
    props: [qinjsCliShared.createBindDirectiveNode("nodes", dir.exp || "")],
    isSelfClosing: true,
    children: [],
    codegenNode: void 0,
    ns: node.ns,
    loc: node.loc
  };
}

const transformText = (node, _) => {
  if (!qinjsCliShared.isElementNode(node)) {
    return;
  }
  const dir = compilerCore.findDir(node, "text");
  if (!dir) {
    return;
  }
  node.props.splice(node.props.indexOf(dir), 1);
  if (node.tagType !== 0) {
    return;
  }
  node.isSelfClosing = false;
  node.children = [
    {
      type: 5,
      loc: dir.exp.loc,
      content: dir.exp
    }
  ];
};

const transformAttr = (node, _) => {
  if (!qinjsCliShared.isElementNode(node)) {
    return;
  }
  node.props.forEach((prop) => {
    if (qinjsCliShared.isAttributeNode(prop) && prop.value) {
      switch (prop.name) {
        case "style":
          prop.value.content = shared.stringifyStyle(
            shared.parseStringStyle(prop.value.content)
          ).slice(0, -1);
          break;
      }
    }
  });
};

function getBaseTransformPreset({
  prefixIdentifiers,
  skipTransformIdentifier
}) {
  const nodeTransforms = [
    transformRoot,
    transformAttr,
    transformTag,
    transformHtml,
    transformText,
    transformIf,
    transformFor,
    transformSlot
  ];
  if (!skipTransformIdentifier) {
    nodeTransforms.push(transformIdentifier);
  }
  nodeTransforms.push(transformElement);
  nodeTransforms.push(transformComponent);
  if (prefixIdentifiers) {
    nodeTransforms.push(transformExpression);
  }
  return [
    nodeTransforms,
    { on: transformOn, bind: transformBind }
  ];
}
function baseCompile(template, options = {}) {
  const prefixIdentifiers = options.prefixIdentifiers === true || options.mode === "module";
  const ast = shared.isString(template) ? compilerCore.baseParse(template, { ...options, parseMode: "html" }) : template;
  const [nodeTransforms, directiveTransforms] = getBaseTransformPreset({
    prefixIdentifiers,
    skipTransformIdentifier: options.skipTransformIdentifier === true
  });
  options.hashId = genHashId(options);
  if (options.filename) {
    if (!options.filters && options.miniProgram?.filter) {
      options.filters = parseFilters(
        options.miniProgram.filter.lang,
        options.filename
      );
    }
  }
  const context = transform(
    ast,
    shared.extend({}, options, {
      prefixIdentifiers,
      nodeTransforms: [...nodeTransforms, ...options.nodeTransforms || []],
      directiveTransforms: shared.extend(
        {},
        directiveTransforms,
        options.directiveTransforms || {}
      )
    })
  );
  const result = shared.extend(
    generate$1(
      shared.extend(ast, {
        bindingComponents: context.bindingComponents
      }),
      options
    ),
    { ast }
  );
  if (options.filename && options.miniProgram?.emitFile) {
    const {
      class: clazz,
      directive,
      emitFile,
      event,
      slot,
      lazyElement,
      component
    } = options.miniProgram;
    generate(ast, {
      class: clazz,
      scopeId: options.scopeId,
      filename: options.filename,
      directive,
      emitFile,
      event,
      slot,
      lazyElement,
      component,
      isBuiltInComponent: context.isBuiltInComponent,
      isMiniProgramComponent: context.isMiniProgramComponent
    });
  }
  return result;
}
function genHashId(options) {
  if (options.hashId) {
    return options.hashId;
  }
  if (options.scopeId) {
    return options.scopeId.replace("data-v-", "");
  }
  if (options.filename) {
    return qinjsCliShared.hash(options.filename);
  }
  return "";
}
function parseFilters(lang, filename) {
  filename = filename.split("?")[0];
  if (fs__default.existsSync(filename)) {
    return qinjsCliShared.parseFilterNames(lang, fs__default.readFileSync(filename, "utf8"));
  }
  return [];
}

const parserOptions = {
  isVoidTag: shared.isVoidTag,
  isNativeTag: (tag) => shared.isHTMLTag(tag) || shared.isSVGTag(tag),
  isPreTag: (tag) => tag === "pre",
  // QINJS 独有
  // [@vue/compiler-core] decodeEntities option is passed but will be ignored in non-browser builds.
  // decodeEntities: decodeHtml,
  // https://html.spec.whatwg.org/multipage/parsing.html#tree-construction-dispatcher
  getNamespace(tag, parent) {
    let ns = parent ? parent.ns : 0 /* HTML */;
    if (parent && ns === 2 /* MATH_ML */) {
      if (parent.tag === "annotation-xml") {
        if (tag === "svg") {
          return 1 /* SVG */;
        }
        if (parent.props.some(
          (a) => a.type === 6 && a.name === "encoding" && a.value != null && (a.value.content === "text/html" || a.value.content === "application/xhtml+xml")
        )) {
          ns = 0 /* HTML */;
        }
      } else if (/^m(?:[ions]|text)$/.test(parent.tag) && tag !== "mglyph" && tag !== "malignmark") {
        ns = 0 /* HTML */;
      }
    } else if (parent && ns === 1 /* SVG */) {
      if (parent.tag === "foreignObject" || parent.tag === "desc" || parent.tag === "title") {
        ns = 0 /* HTML */;
      }
    }
    if (ns === 0 /* HTML */) {
      if (tag === "svg") {
        return 1 /* SVG */;
      }
      if (tag === "math") {
        return 2 /* MATH_ML */;
      }
    }
    return ns;
  },
  parseMode: "html"
};

function parse(template, options = {}) {
  return compilerCore.baseParse(template, shared.extend({}, parserOptions, options));
}
function beforeCompile(theDir) {
  return function compile(template, options = {}) {
    return baseCompile(
      template,
      shared.extend({}, parserOptions, options, {
        // QINJS 独有 注入环境变量或配置
        dir: theDir,
        directiveTransforms: shared.extend({}, options.directiveTransforms || {})
      })
    );
  };
}

exports.findProp = compilerCore.findProp;
exports.CAMELIZE = CAMELIZE;
exports.DYNAMIC_SLOT = DYNAMIC_SLOT;
exports.EXTEND = EXTEND;
exports.HYPHENATE = HYPHENATE;
exports.NORMALIZE_CLASS = NORMALIZE_CLASS;
exports.RENDER_PROPS = RENDER_PROPS;
exports.RENDER_SLOT = RENDER_SLOT;
exports.SET_REF = SET_REF;
exports.STRINGIFY_STYLE = STRINGIFY_STYLE;
exports.TO_DISPLAY_STRING = TO_DISPLAY_STRING;
exports.V_FOR = V_FOR;
exports.V_ON = V_ON;
exports.WITH_MODEL_MODIFIERS = WITH_MODEL_MODIFIERS;
exports.WITH_SCOPED_SLOT = WITH_SCOPED_SLOT;
exports.beforeCompile = beforeCompile;
exports.genExpr = genExpr;
exports.isForElementNode = isForElementNode;
exports.parse = parse;
exports.rewriteExpression = rewriteExpression;
exports.transformModel = transformModel;
exports.transformOn = transformOn;
